# -*- coding: utf-8 -*-

# Copyright(C) 2016      Phyks
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.browser.pages import HTMLPage, LoggedPage
from weboob.browser.elements import method, ItemElement, ListElement
from weboob.browser.filters.standard import CleanDecimal, CleanText
from weboob.browser.filters.standard import DateTime, Env, Eval, Format
from weboob.browser.filters.html import Link
from weboob.capabilities.calendar import CATEGORIES, TICKET

from .calendar import CentQuatreEvent
from datetime import datetime, timedelta


class CentQuatrePage(HTMLPage):
    @property
    def logged(self):
        return bool(self.doc.xpath(u'//*[@id="account_logout"]'))


class LoginPage(CentQuatrePage):
    def login(self, email, password):
        form = self.get_form(id='login_form')
        form['login'] = email
        form['password'] = password
        form.submit()

class TicketsPage(CentQuatrePage, LoggedPage):
    def list_tickets(self, date_from=None, date_to=None):
        tickets_containers = self.doc.xpath(u'//*[@class="product_container"]')
        tickets = []
        for ticket_container in tickets_containers:
            date = datetime.strptime(
                ticket_container.xpath(u'//*[@class="day"]')[0].text.strip(),
                u'%A, %d %B %Y - %H:%M'
            )
            if date_from and date < date_from:
                continue
            if date_to and date > date_to:
                continue
            tickets.append(
                ticket_container.xpath(
                    u'//*[@class="file_number"]/a'
                )[0].text.strip().split(' ')[1]
            )
        return tickets


class TicketsDetailsPage(CentQuatrePage, LoggedPage):
    @method
    class get_event_details(ListElement):
        item_xpath = u'//*[@class="product_container"]'

        class EventDetails(ItemElement):
            klass = CentQuatreEvent

            obj_id = Env('fileId')
            obj_start_date = DateTime(
                CleanText(u'//*[@class="date"]')
            )
            obj_end_date = Eval(
                lambda x: x + timedelta(hours=1),
                obj_start_date
            )
            obj_timezone = u'Europe/Paris'
            obj_summary = CleanText(
                u'//*[@class="content_product_info"]//*[contains(@class, "title")]'
            )
            obj_city = u'Paris'
            obj_location = Format(
                "%s, %s",
                CleanText(
                    u'(//*[@class="location"])[1]'
                ),
                CleanText(
                    u'//*[@class="address"]'
                )
            )
            obj_category = CATEGORIES.SPECTACLE
            obj_price = CleanDecimal(
                CleanText(
                    u'(//*[@class="unit_price with_beneficiary"])[1]'
                )
            )
            obj_description = Format(
                u'%s. %s. %.2f€.',
                CleanText(
                    u'(//*[contains(@class, "tariff") and contains(@class, "with_beneficiary")])[1]'
                ),
                CleanText(
                    u'(//*[contains(@class, "seat") and contains(@class, "with_beneficiary")])[1]'
                ),
                obj_price,
            )
            obj_ticket = TICKET.AVAILABLE
            def obj_url(self):
                return (
                    u'%s%s' % (
                        self.page.browser.BASEURL,
                        Link(
                            u'//*[@class="alternative_button mticket"]/a'
                        )(self)
                    )
                )



# -*- coding: utf-8 -*-

# Copyright(C) 2015 Guilhem Bonnefille
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.capabilities.video import CapVideo, BaseVideo
from weboob.capabilities.collection import CapCollection, Collection
from weboob.tools.backend import Module

from .browser import RmllBrowser
from .video import RmllVideo


__all__ = ['RmllModule']


class RmllModule(Module, CapVideo, CapCollection):
    NAME = 'rmll'                          # The name of module
    MAINTAINER = u'Guyou'                  # Name of maintainer of this module
    EMAIL = 'guilhem.bonnefille@gmail.com' # Email address of the maintainer
    VERSION = '1.3'                        # Version of weboob
    DESCRIPTION = 'Videos from RMLL'       # Description of your module
    LICENSE = 'AGPLv3+'                    # License of your module

    BROWSER = RmllBrowser

    def create_default_browser(self):
        return self.create_browser()

    def get_video(self, _id):
        self.logger.debug("Getting video for %s", _id)
        return self.browser.get_video(_id)

    def search_videos(self, pattern, sortby=CapVideo.SEARCH_RELEVANCE, nsfw=False):
        return self.browser.search_videos(pattern)

    def fill_video(self, video, fields):
        self.logger.debug("Fill video %s for fields %s", video.id, fields)
        if fields != ['thumbnail']:
            # if we don't want only the thumbnail, we probably want also every fields
            video = self.browser.get_video(video.id, video)
        if 'thumbnail' in fields and video and video.thumbnail:
            video.thumbnail.data = self.browser.open(video.thumbnail.url).content

        return video

    def iter_resources(self, objs, split_path):
        if BaseVideo in objs:
            if len(split_path) == 0:
                # Add fake Collection
                yield Collection(['latest'], u'Latest')
            if len(split_path) == 1 and split_path[0] == 'latest':
                for video in self.browser.get_latest_videos():
                    yield video
            else:
                channel_videos = self.browser.get_channel_videos(split_path)
                if channel_videos:
                    for content in channel_videos:
                        yield content

    OBJECTS = {RmllVideo: fill_video}



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from datetime import timedelta

from weboob.capabilities.image import Thumbnail
from weboob.capabilities.base import BaseObject, NotAvailable
from weboob.capabilities.collection import Collection
from weboob.capabilities.base import empty
from weboob.browser.pages import HTMLPage, JsonPage, pagination
from weboob.browser.elements import DictElement, ItemElement, ListElement, method
from weboob.browser.filters.standard import Date, Format, Env, CleanText, Field, Regexp, Join, Eval
from weboob.browser.filters.json import Dict
from weboob.browser.filters.html import XPath
from weboob.tools.compat import basestring, unquote

from .video import ArteVideo, ArteSiteVideo, SITE


class ArteItemElement(ItemElement):

    def condition(self):
        return 'VID' in self.el

    obj_id = Dict('VID')

    def obj_title(self):
        vti = Dict('VTI')(self)
        vtu = Dict('VSU', default=None)(self)
        if not vtu:
            return vti

        return '%s: %s' % (vti, vtu)

    obj_rating = Dict('VRT', default=NotAvailable)
    obj_rating_max = 10
    obj_description = Dict('VDE', default=NotAvailable)
    obj_date = Date(Dict('VDA', default=NotAvailable), default=NotAvailable)

    def obj_duration(self):
        seconds = Dict('videoDurationSeconds', default=NotAvailable)(self)
        if empty(seconds):
            return seconds
        elif isinstance(seconds, basestring):
            seconds = int(seconds)

        return timedelta(seconds=seconds)

    def obj_thumbnail(self):
        url = Dict('VTU/IUR', default=NotAvailable)(self)
        if empty(url):
            return url

        thumbnail = Thumbnail(url)
        thumbnail.url = thumbnail.id
        return thumbnail


class VideosListPage(HTMLPage):

    @method
    class iter_arte_concert_categories(ListElement):
        item_xpath = '//ul[@class="filter-liste"]/li'

        class item(ItemElement):
            klass = Collection

            obj_title = CleanText('./a/span')
            obj_id = CleanText('./@data-target', replace=[('video_box_tab_', '')])

            def obj_split_path(self):
                _id = CleanText('./@data-target', replace=[('video_box_tab_', '')])(self)
                return [SITE.CONCERT.get('id'), u'%s' % _id]

    @method
    class iter_arte_concert_videos(ListElement):

        def find_elements(self):
            self.item_xpath = '//div[@id="video_box_tab_%s"]/article' % Env('cat')(self)
            for el in self.el.xpath(self.item_xpath):
                yield el

        class item(ItemElement):
            klass = ArteSiteVideo

            obj__site = SITE.CONCERT.get('id')
            obj_id = Format('%s.%s', Field('_site'), CleanText('./@about'))
            obj_title = CleanText('div/div[@class="info-article "]/div/h3/a')

            def obj_thumbnail(self):
                url = CleanText('div/div/a/figure/span/span/@data-src')(self)
                thumbnail = Thumbnail(url)
                thumbnail.url = thumbnail.id
                return thumbnail

    @method
    class iter_arte_creative_categories(ListElement):
        item_xpath = '//ul[@class="menu"]/li/a[not(@target)]'

        class item(ItemElement):
            klass = Collection

            obj_title = CleanText('.', default=u'Accueil')
            obj_id = CleanText('./@href')

            def obj_split_path(self):
                _id = Regexp(CleanText('./@href'), '/\w{2}/(.*)', default=u'accueil')(self)
                return [SITE.CREATIVE.get('id')] + [_id.replace('/', '^')]

    @method
    class iter_arte_creative_videos(ListElement):
        item_xpath = '//div[div/i]'
        ignore_duplicate = True

        class item(ItemElement):
            klass = ArteSiteVideo

            obj__site = SITE.CREATIVE.get('id')
            obj_id = Format('%s.%s', Field('_site'),
                            CleanText('./div/h3/a/@href|./div/h1/a/@href'))
            obj_title = CleanText('./div/h3/a|./div/h1/a')

            def obj_thumbnail(self):
                url = CleanText('./div/a/img/@src')(self)
                thumbnail = Thumbnail(url)
                thumbnail.url = thumbnail.id
                return thumbnail

    @method
    class iter_arte_cinema_categories(ListElement):
        item_xpath = '//li[has-class("leaf")]'

        class item(ItemElement):
            klass = Collection

            def condition(self):
                return Regexp(CleanText('./a/@href'), '^(/\w{2}/%s/.*)' % self.env['cat'], default=None)(self)

            obj_title = CleanText('./a')
            obj_id = CleanText('./a/@href')

            def obj_split_path(self):
                _id = Regexp(CleanText('./a/@href'), '/\w{2}/(.*)')(self)
                return [SITE.CINEMA.get('id')] + _id.split('/')

    def get_arte_cinema_menu(self):
        return self.doc.xpath('//li[has-class("leaf")]/a[starts-with(@href,"/")]/@href')

    @method
    class get_arte_cinema_videos(ListElement):
        item_xpath = '//div[has-class("article-list")]/article[@id]'

        class item(ItemElement):
            klass = ArteSiteVideo

            def condition(self):
                return len(XPath('.//div[@class="article-secondary "]')(self)) == 1

            obj__site = SITE.CINEMA.get('id')
            obj_id = Format('%s.%s', Field('_site'),
                            Regexp(CleanText('./div/div/a/@href|./div/a/@href'),
                                   '(http://.*\.arte\.tv)?/(.*)',
                                   '\\2'))
            obj_title = Join(u' - ',
                             './/div[@class="article-secondary "]/div/div')

            def obj_thumbnail(self):
                url = CleanText('.//div[@class="article-primary "]/div[has-class("field-thumbnail")]/span/noscript/img/@src')(self)
                thumbnail = Thumbnail(url)
                thumbnail.url = thumbnail.id
                return thumbnail

    def get_json_url(self):
        if self.doc.xpath('//div[@class="video-container"]'):
            return self.doc.xpath('//div[@class="video-container"]')[0].attrib['arte_vp_url']
        elif self.doc.xpath('//iframe'):
            url = Regexp(CleanText('./@src'), '.*json_url=(.*)', default='')(self.doc.xpath('//iframe')[0])
            return unquote(url)
        return ''


class ArteJsonPage(JsonPage):

    def get_video_url(self, quality, format, version, language_version):
        _urls = Dict('videoJsonPlayer/VSR')(self.doc)
        if _urls:
            urls = _urls.keys()
            key = '_'.join([format, quality, version])
            found = self.find_url(key, urls, version, quality)
            if not found:
                # We use the default language version
                key = '_'.join([format, quality, language_version])
                found = self.find_url(key, urls, version, quality)
                if not found:
                    # We only keep the quality
                    key = '_'.join([quality, language_version])
                    found = self.find_url(key, urls, version, quality)
                    if not found:
                        found = urls[0]
            streamer = Dict('videoJsonPlayer/VSR/%s/streamer' % (found), default=None)(self.doc)
            url = Dict('videoJsonPlayer/VSR/%s/url' % (found))(self.doc)
            if streamer:
                return '%s%s' % (streamer, url), found
            return url, found
        return NotAvailable, ''

    def find_url(self, key, urls, version, quality):
        self.logger.debug('available urls: %s' % urls)
        self.logger.debug('search url matching : %s' % key)
        # Best Case: key is mathing
        matching = [s for s in urls if key in s]
        self.logger.debug('best case matching: %s' % matching)
        if matching:
            return matching[0]

        # Second Case: is the version available
        matching = [s for s in urls if version in s]
        self.logger.debug('is version available: %s' % matching)
        if matching:
            # Do the quality + version match
            matching_quality = [s for s in matching if quality in s]
            self.logger.debug('does quality + version match: %s' % matching_quality)
            if matching_quality:
                return matching[0]

            # Only format + version mathes
            return matching[0]

    @method
    class iter_videos(DictElement):
        item_xpath = 'videoList'

        class item(ArteItemElement):
            klass = ArteVideo

    @method
    class iter_programs(DictElement):
        item_xpath = 'configClusterList'

        class item(ItemElement):
            klass = Collection

            obj_title = Dict(CleanText(Env('title')))
            obj_id = Dict('clusterId')

            def obj_split_path(self):
                return [SITE.PROGRAM.get('id'), Dict('clusterId')(self)]

    @method
    class get_video(ArteItemElement):
        def __init__(self, *args, **kwargs):
            super(ArteItemElement, self).__init__(*args, **kwargs)
            self.el = self.el.get('videoJsonPlayer')

        klass = ArteVideo

    @method
    class get_arte_concert_video(ArteItemElement):
        def __init__(self, *args, **kwargs):
            super(ArteItemElement, self).__init__(*args, **kwargs)
            self.el = self.el.get('videoJsonPlayer')

        klass = ArteSiteVideo
        obj__site = SITE.CONCERT.get('id')

    @method
    class get_arte_cinema_video(ArteItemElement):
        def __init__(self, *args, **kwargs):
            super(ArteItemElement, self).__init__(*args, **kwargs)
            self.el = self.el.get('videoJsonPlayer')

        klass = ArteSiteVideo

        obj__site = SITE.CINEMA.get('id')
        obj_date = Date(Dict('VRA', default=''), default=NotAvailable)

    @method
    class get_program_video(ArteItemElement):
        def __init__(self, *args, **kwargs):
            super(ArteItemElement, self).__init__(*args, **kwargs)

            if 'videoJsonPlayer' in self.el:
                self.el = self.el.get('videoJsonPlayer')
            elif 'abstractProgram' not in self.el:
                return None
            elif 'VDO' in self.el['abstractProgram'].keys():
                self.el = self.el['abstractProgram']['VDO']

        klass = ArteVideo

    @method
    class iter_program_videos(DictElement):
        item_xpath = 'clusterWrapper/broadcasts'
        ignore_duplicate = True

        class item(ItemElement):
            klass = BaseObject

            def condition(self):
                return 'VDS' in self.el.keys() and len(self.el['VDS']) > 0

            obj_id = Dict('programId')


class SearchPage(JsonPage):
    @pagination
    @method
    class iter_videos(DictElement):
        item_xpath = 'teasers'

        class item(ItemElement):
            klass = ArteVideo

            obj_id = Dict('id')
            obj_title = Dict('title')
            obj_duration = Eval(lambda x: x * 60, Dict('duration'))
            obj_date = Date(Dict('creationDate'))

            def obj_thumbnail(self):
                try:
                    return Thumbnail(next(img['url'] for img in self.el['images'] if img['format'] == 'landscape'))
                except StopIteration:
                    return NotAvailable

        def next_page(self):
            if self.el['metas']['page'] < self.el['metas']['pages']:
                return self.page.url.rstrip('0123456789') + str(self.el['metas']['page'] + 1)



# -*- coding: utf-8 -*-

# Copyright(C) 2016      Edouard Lambert
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


import re

from weboob.browser.pages import HTMLPage, LoggedPage, pagination
from weboob.browser.filters.standard import CleanText, CleanDecimal, Env, Format, TableCell, Date, Async, AsyncLoad
from weboob.browser.elements import ListElement, ItemElement, TableElement, method
from weboob.browser.filters.html import Attr
from weboob.capabilities.bill import Bill, Subscription
from weboob.capabilities.base import NotAvailable


class LoginPage(HTMLPage):
    def login(self, login, password):
        form = self.get_form('//form[contains(@action, "login")]')
        form['login'] = login
        form['pass'] = password
        form.submit()

    def get_error(self):
        return CleanText('//div[@class="ValidatorError"]')(self.doc)

class CaptchaPage(HTMLPage):
    def get_error(self):
        return CleanText('//div[@class="captcha-block"]/p[1]/text()')(self.doc)

class ProfilPage(LoggedPage, HTMLPage):
    @method
    class get_list(ListElement):
        class item(ItemElement):
            klass = Subscription

            obj_subscriber = Format('%s %s', Attr('//input[@id="prenom"]', 'value'), Attr('//input[@id="nom"]', 'value'))
            obj_id = Env('subid')
            obj_label = obj_id

            def parse(self, el):
                self.env['subid'] = self.page.browser.username

class DocumentsPage(LoggedPage, HTMLPage):
    @pagination
    @method
    class get_documents(TableElement):
        item_xpath = '//table[@class="EpCmdList"]/tr[@class="Line1" or @class="Line2"]'
        head_xpath = '//table[@class="EpCmdList"]/tr/th'

        col_id = u'Référence'
        col_date = u'Date'
        col_price = u'Montant'

        def next_page(self):
            m = re.search('([^*]+page=)([^*]+)', self.page.url)
            if m:
                page = int(m.group(2)) + 1
                if self.el.xpath('//a[contains(@href, "commande.html?page=' + str(page) + '")]'):
                    next_page = u"%s%s" % (m.group(1), page)
                    return next_page

        class item(ItemElement):
            klass = Bill

            load_details = Attr('./td/a', 'href') & AsyncLoad

            obj_id = Format('%s_%s', Env('email'), CleanDecimal(TableCell('id')))
            obj__url = Async('details') & Attr('//a[contains(@href, "facture")]', 'href', default=NotAvailable)
            obj_date = Date(CleanText(TableCell('date')))
            obj_format = u"pdf"
            obj_label = Async('details') & CleanText('//table/tr/td[@class="Prod"]')
            obj_type = u"bill"
            obj_price = CleanDecimal(TableCell('price'), replace_dots=True)
            obj_currency = u"€"

            def parse(self, el):
                self.env['email'] = self.page.browser.username



# -*- coding: utf-8 -*-

# Copyright(C) 2013 Roger Philibert
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.deprecated.browser import Browser
from weboob.deprecated.browser.decorators import id2url
from weboob.tools.compat import quote

from .video import JacquieEtMichelVideo
from .pages import VideoPage, ResultsPage


__all__ = ['JacquieEtMichelBrowser']


class JacquieEtMichelBrowser(Browser):
    DOMAIN = u'jacquieetmicheltv.net'
    ENCODING = None
    PAGES = {r'https?://.*jacquieetmicheltv.net/': ResultsPage,
             r'https?://.*jacquieetmicheltv.net/videolist/.*': ResultsPage,
             r'https?://.*jacquieetmicheltv.net/showvideo/(?P<id>\d+)/.*': VideoPage,
            }

    @id2url(JacquieEtMichelVideo.id2url)
    def get_video(self, url, video=None):
        self.location(url)
        assert self.is_on_page(VideoPage), 'Should be on video page.'
        return self.page.get_video(video)

    def search_videos(self, pattern):
        self.location('/videolist/searchmodevideo/query%s/' % (quote(pattern.encode('utf-8'))))
        assert self.is_on_page(ResultsPage)
        return self.page.iter_videos()

    def latest_videos(self):
        self.home()
        assert self.is_on_page(ResultsPage)
        return self.page.iter_videos()



# -*- coding: utf-8 -*-

# Copyright(C) 2013 Julien Veyssier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import requests

from PyQt5.QtCore import pyqtSlot as Slot, Qt
from PyQt5.QtGui import QImage, QPixmap
from PyQt5.QtWidgets import QFrame, QApplication, QMessageBox

from weboob.applications.qcineoob.ui.person_ui import Ui_Person
from weboob.capabilities.base import empty


class Person(QFrame):
    def __init__(self, person, backend, parent=None):
        super(Person, self).__init__(parent)
        self.parent = parent
        self.ui = Ui_Person()
        self.ui.setupUi(self)

        self.ui.filmographyButton.clicked.connect(self.filmography)
        self.ui.biographyButton.clicked.connect(self.biography)
        self.ui.moviesInCommonButton.clicked.connect(self.moviesInCommon)

        self.person = person
        self.backend = backend
        self.gotThumbnail()
        self.ui.nameLabel.setText(person.name)

        self.ui.idEdit.setText(u'%s@%s' % (person.id, backend.name))
        if not empty(person.real_name):
            self.ui.realNameLabel.setText('%s' % person.real_name)
        else:
            self.ui.realNameLabel.parent().hide()
        if not empty(person.birth_place):
            self.ui.birthPlaceLabel.setText('%s' % person.birth_place)
        else:
            self.ui.birthPlaceLabel.parent().hide()
        if not empty(person.birth_date):
            self.ui.birthDateLabel.setText(person.birth_date.strftime('%Y-%m-%d'))
        else:
            self.ui.birthDateLabel.parent().hide()
        if not empty(person.death_date):
            self.ui.deathDateLabel.setText(person.death_date.strftime('%Y-%m-%d'))
        else:
            self.ui.deathDateLabel.parent().hide()
        self.ui.shortBioPlain.setPlainText('%s' % person.short_biography)
        for role in person.roles.keys():
            self.ui.filmographyCombo.addItem(role)
        self.ui.verticalLayout_2.setAlignment(Qt.AlignTop)

    def gotThumbnail(self):
        if not empty(self.person.thumbnail_url):
            data = requests.get(self.person.thumbnail_url).content
            img = QImage.fromData(data)
            self.ui.imageLabel.setPixmap(QPixmap.fromImage(img).scaledToWidth(220,Qt.SmoothTransformation))

    @Slot()
    def filmography(self):
        role = None
        tosearch = self.ui.filmographyCombo.currentText()
        role_desc = ''
        if tosearch != 'all':
            role = tosearch
            role_desc = ' as %s' % role
        self.parent.doAction('Filmography of "%s"%s' % (self.person.name, role_desc),
                             self.parent.filmographyAction, [self.backend.name, self.person.id, role])

    @Slot()
    def biography(self):
        QApplication.setOverrideCursor(Qt.WaitCursor)
        self.backend.fill_person(self.person, 'biography')
        bio = self.person.biography
        self.ui.shortBioPlain.setPlainText(u'%s' % bio)
        self.ui.biographyLabel.setText('Full biography:')
        self.ui.biographyButton.hide()
        QApplication.restoreOverrideCursor()

    @Slot()
    def moviesInCommon(self):
        my_id = self.person.id
        my_name = self.person.name
        other_id = self.ui.moviesInCommonEdit.text().split('@')[0]
        other_person = self.backend.get_person(other_id)
        if other_id == self.person.id:
            QMessageBox.critical(None, self.tr('"Moviess in common" error'),
                                 self.tr('Nice try\nThe persons must be different'),
                                 QMessageBox.Ok)
        elif not other_person:
            QMessageBox.critical(None, self.tr('"Movies in common" error'),
                                 self.tr('Person not found: %s' % other_id),
                                 QMessageBox.Ok)
        else:
            other_name = other_person.name
            desc = 'Movies in common %s, %s'%(my_name, other_name)
            self.parent.doAction(desc, self.parent.moviesInCommonAction, [self.backend.name, my_id, other_id])



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2012  Jeremy Monnet
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.capabilities.library import CapBook
from weboob.tools.backend import Module, BackendConfig
from weboob.tools.value import ValueBackendPassword, Value

from .browser import AloesBrowser


__all__ = ['AloesModule']


class AloesModule(Module, CapBook):
    NAME = 'opacwebaloes'
    MAINTAINER = u'Jeremy Monnet'
    EMAIL = 'jmonnet@gmail.com'
    VERSION = '1.3'
    DESCRIPTION = 'Aloes Library software'
    LICENSE = 'AGPLv3+'
    CONFIG = BackendConfig(Value('login',    label='Account ID', regexp='^\d{1,8}\w$'),
                           ValueBackendPassword('password', label='Password of account'),
                           Value('baseurl',    label='Base URL')
                           )
    BROWSER = AloesBrowser

    def create_default_browser(self):
        return self.create_browser(self.config['baseurl'].get(),
                                   self.config['login'].get(),
                                   self.config['password'].get())

    def iter_rented(self):
        for book in self.browser.get_rented_books_list():
            yield book

    def iter_booked(self):
        for book in self.browser.get_booked_books_list():
            yield book

    def iter_books(self):
        for book in self.iter_booked():
            yield book
        for book in self.iter_rented():
            yield book

    def get_book(self, _id):
        raise NotImplementedError()

    def search_books(self, _string):
        raise NotImplementedError()



#!/usr/bin/env python3

"""
This script edits your backends conf file by replacing stuff like:

[bnporc21]
_module = bnporc
website = pp
login = 123456
password = 78910

with:

[bnporc21]
_module = bnporc
website = pp
login = 123456
password = `pass show weboob/bnporc21`
"""

from __future__ import print_function

import os
import re
import shutil
import subprocess
import sys
import tempfile

FILE = os.getenv('WEBOOB_BACKENDS') or os.path.expanduser('~/.config/weboob/backends')

if not os.path.exists(FILE):
    print('the backends file does not exist')
    sys.exit(os.EX_NOINPUT)

if not shutil.which('pass'):
    print('the "pass" tool could not be found')
    sys.exit(os.EX_UNAVAILABLE)

errors = 0
seen = set()

backend = None
with open(FILE) as inp:
    with tempfile.NamedTemporaryFile('w', delete=False, dir=os.path.dirname(FILE)) as outp:
        for line in inp:
            line = line.strip()

            mtc = re.match(r'password\s*=\s*(\S.*)$', line)
            if mtc and not mtc.group(1).startswith('`'):
                cmd = ['pass', 'insert', 'weboob/%s' % backend]
                stdin = 2 * ('%s\n' % mtc.group(1))

                proc = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
                proc.communicate(stdin.encode('utf-8'))
                if proc.returncode == 0:
                    print('password = `pass show weboob/%s`' % backend, file=outp)
                    continue
                else:
                    errors += 1
                    print('warning: could not store password for backend %r' % backend)

            mtc = re.match(r'\[(.+)\]', line)
            if mtc:
                backend = mtc.group(1)
                if backend in seen:
                    print('error: backend %r is present multiple times' % backend)
                    sys.exit(os.EX_DATAERR)
                seen.add(backend)

            print(line, file=outp)

os.rename(outp.name, FILE)

if errors:
    print('%d errors were encountered when storing passwords securely' % errors)
    sys.exit(2)



# -*- coding: utf-8 -*-

# Copyright(C) 2013      Vincent A
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from ctypes import cdll, c_char_p, c_double, c_void_p, byref
from ctypes.util import find_library


class QuviError(Exception):
    pass


class LibQuvi04(object):
    QUVI_VERSION = 0

    QUVIOPT_FORMAT = 0
    QUVIOPT_CATEGORY = 4
    QUVI_OK = 0
    QUVI_LAST = 5

    QUVIPROP_PAGEURL = 0x100002
    QUVIPROP_PAGETITLE = 0x100003
    QUVIPROP_MEDIAID = 0x100004
    QUVIPROP_MEDIAURL = 0x100005
    QUVIPROP_FILESUFFIX = 0x100008
    #~ QUVIPROP_FORMAT = 0x10000A
    QUVIPROP_MEDIATHUMBNAILURL = 0x10000C
    QUVIPROP_MEDIACONTENTLENGTH = 0x300006
    QUVIPROP_MEDIADURATION = 0x30000D

    QUVIPROTO_HTTP = 1
    QUVIPROTO_RTMP = 8

    def __init__(self, lib=None):
        self.lib = lib
        self.qh = c_void_p()
        self.qmh = c_void_p()

    def load(self):
        path = find_library('quvi')
        if not path:
            return False

        self.lib = cdll.LoadLibrary(path)
        if self.lib is None:
            return False

        self.lib.quvi_version.restype = c_char_p
        version_str = self.lib.quvi_version(self.QUVI_VERSION)
        if version_str.startswith('v0.4'):
            return True
        else:
            return False

    def _cleanup(self):
        if self.qmh:
            self.lib.quvi_parse_close(byref(self.qmh))
            self.qmh = c_void_p()
        if self.qh:
            self.lib.quvi_close(byref(self.qh))
            self.qh = c_void_p()

    def get_info(self, url):
        try:
            return self._get_info(url)
        finally:
            self._cleanup()

    def _get_info(self, url):
        status = self.lib.quvi_init(byref(self.qh))
        self._assert_ok(status)

        status = self.lib.quvi_setopt(self.qh, self.QUVIOPT_FORMAT, 'best')
        self._assert_ok(status)

        status = self.lib.quvi_parse(self.qh, c_char_p(url), byref(self.qmh))
        self._assert_ok(status)

        info = {}
        info['url'] = self._get_str(self.QUVIPROP_MEDIAURL)
        info['title'] = self._get_str(self.QUVIPROP_PAGETITLE)
        info['suffix'] = self._get_str(self.QUVIPROP_FILESUFFIX)
        info['page'] = self._get_str(self.QUVIPROP_PAGEURL) # uncut!
        info['media_id'] = self._get_str(self.QUVIPROP_MEDIAID)
        info['thumbnail'] = self._get_str(self.QUVIPROP_MEDIATHUMBNAILURL)
        info['duration'] = self._get_double(self.QUVIPROP_MEDIADURATION)
        info['size'] = self._get_double(self.QUVIPROP_MEDIACONTENTLENGTH)

        return info

    def _assert_ok(self, status):
        if status != self.QUVI_OK:
            self.lib.quvi_strerror.restype = c_char_p
            c_msg = self.lib.quvi_strerror(self.qh, status)
            raise QuviError(c_msg)

    def _get_str(self, prop):
        c_value = c_char_p()
        status = self.lib.quvi_getprop(self.qmh, prop, byref(c_value))
        self._assert_ok(status)

        return c_value.value

    def _get_double(self, prop):
        c_value = c_double()
        status = self.lib.quvi_getprop(self.qmh, prop, byref(c_value))
        self._assert_ok(status)

        return c_value.value


LibQuvi = LibQuvi04



# -*- coding: utf-8 -*-

# Copyright(C) 2013 Sébastien Monel
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.capabilities.job import CapJob
from weboob.tools.application.qt5 import QtApplication
from weboob.tools.config.yamlconfig import YamlConfig

from .main_window import MainWindow


class QHandJoob(QtApplication):
    APPNAME = 'qhandjoob'
    VERSION = '1.3'
    COPYRIGHT = u'Copyright(C) 2013-2014 Sébastien Monel'
    DESCRIPTION = "Qt application to search for job."
    SHORT_DESCRIPTION = "search for job"
    CAPS = CapJob
    CONFIG = {'queries': {}}
    STORAGE = {'bookmarks': [], 'read': [], 'notes': {}}

    def main(self, argv):
        self.load_backends(CapJob)
        self.create_storage()
        self.load_config(klass=YamlConfig)

        main_window = MainWindow(self.config, self.storage, self.weboob)
        main_window.show()
        return self.weboob.loop()



# -*- coding: utf-8 -*-

# Copyright(C) 2015 Matthieu Weber
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.browser.browsers import PagesBrowser
from weboob.browser.url import URL
from .pages import WeatherPage, SearchCitiesPage, ObservationsPage

__all__ = ['IlmatieteenlaitosBrowser']


class IlmatieteenlaitosBrowser(PagesBrowser):
    BASEURL = 'http://ilmatieteenlaitos.fi'
    cities = URL('/etusivu\?p_p_id=locationmenuportlet_WAR_fmiwwwweatherportlets&p_p_lifecycle=2&p_p_state=normal&'
                 'p_p_mode=view&p_p_cacheability=cacheLevelFull&term=(?P<pattern>.*)', SearchCitiesPage)
    weather_query = URL('/paikallissaa\?p_p_id=locationmenuportlet_WAR_fmiwwwweatherportlets&p_p_lifecycle=1&'
                        'p_p_state=normal&p_p_mode=view&_locationmenuportlet_WAR_fmiwwwweatherportlets_action='
                        'changelocation')
    weather = URL('/saa/(?P<city_url>.*)', WeatherPage)
    observations = URL('/observation-data\?station=(?P<station_id>.*)', ObservationsPage)

    def iter_city_search(self, pattern):
        return self.cities.go(pattern=pattern).iter_cities()

    def iter_forecast(self, city):
        return self.weather_query.go(data={"place": city.name, "forecast": "short"}).iter_forecast()

    def get_current(self, city):
        station_id = self.weather_query.go(data={"place": city.name, "forecast": "short"}).get_station_id()
        return self.observations.go(station_id=station_id).get_current()



# -*- coding: utf-8 -*-

# Copyright(C) 2014      Alexandre Morignot
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.capabilities.calendar import CATEGORIES, STATUS, TICKET
from weboob.browser.elements import ItemElement, ListElement, method
from weboob.browser.filters.html import Attr, CleanHTML, Link
from weboob.browser.filters.standard import CleanDecimal, CleanText, Date, CombineDate, DateTime, Regexp, Time, Type
from weboob.browser.pages import HTMLPage
from weboob.capabilities.calendar import BaseCalendarEvent

from datetime import timedelta


class BasePage(HTMLPage):
    @property
    def logged(self):
        return bool(self.doc.xpath('//li[@id="profile"]/span[contains(text(), "Welcome")]'))


class LoginPage(BasePage):
    def login(self, username, password):
        form = self.get_form()
        form['UsernameOrEmailAddress'] = username
        form['Password'] = password
        form.submit()


class ListPage(BasePage):
    @method
    class get_events(ListElement):
        item_xpath = '//ul[@id="items"]/li/article'

        class item(ItemElement):
            klass = BaseCalendarEvent

            obj_url = Link('./div[@class="bbox"]/h1/a')
            obj_id = Regexp(Link('./div[@class="bbox"]/h1/a'), r'aspx\?(.+)')
            obj_location = CleanText('./div[@class="bbox"]/span/a')
            obj_start_date = DateTime(Attr('.//time', 'datetime'))
            obj_summary = Regexp(Attr('./div[@class="bbox"]/h1/a', 'title'), r'details of (.+)')
            obj_category = CATEGORIES.CONCERT
            obj_status = STATUS.CONFIRMED

    def get_country_id(self, country):
        return Regexp(Link('//li[@id="liCountry"]/ul/li/a[./text()="%s"]' % country, default=''), r'ai=([^&]+)&?', default=None)(self.doc)

    def get_city_id(self, city):
        return Regexp(Link('//li[@id="liArea"]/ul/li/a[./text()="%s"]' % city, default=''), r'ai=([^&]+)&?', default=None)(self.doc)

    def get_country_id_next_to(self, country_id):
        return Regexp(Link('//li[@id="liCountry"]/ul/li[./a[contains(@href, "ai=%s&")]]/following-sibling::li/a' % country_id, default=''), r'ai=([^&]+)&?', default=None)(self.doc)


class EventPage(BasePage):
    @method
    class get_event(ItemElement):
        klass = BaseCalendarEvent

        obj_summary = CleanText('//div[@id="sectionHead"]/h1')
        obj_description = CleanHTML('//div[@id="event-item"]/div[3]/p[2]')
        obj_price = CleanDecimal(Regexp(CleanText('//aside[@id="detail"]/ul/li[3]'), r'Cost /[^\d]*([\d ,.]+).', default=''), default=None)
        obj_location = Regexp(CleanText('//aside[@id="detail"]/ul/li[2]'), r'Venue / (.+)')
        obj_booked_entries = Type(CleanText('//h1[@id="MembersFavouriteCount"]'), type=int)
        obj_status = STATUS.CONFIRMED
        obj_category = CATEGORIES.CONCERT

        _date = Date(CleanText('//aside[@id="detail"]/ul/li[1]/a[1]'))

        def obj_start_date(self):
            start_time = Time(Regexp(CleanText('//aside[@id="detail"]/ul/li[1]'), r'(\d{2}:\d{2}) -'))(self)
            return CombineDate(self._date, start_time)(self)

        def obj_end_date(self):
            end_time = Time(Regexp(CleanText('//aside[@id="detail"]/ul/li[1]'), r'- (\d{2}:\d{2})'))(self)

            end_date = CombineDate(self._date, end_time)(self)
            if end_date > self.obj_start_date():
                end_date += timedelta(days = 1)

            return end_date

        def obj_ticket(self):
            li_class = Attr('//li[@id="tickets"]//li[1]', 'class', default=None)(self)

            if li_class:
                if li_class == 'closed':
                    return TICKET.CLOSED
                else:
                    return TICKET.AVAILABLE

            return TICKET.NOTAVAILABLE


class SearchPage(BasePage):
    @method
    class get_events(ListElement):
        item_xpath = '//main/ul/li/section/div/div/ul/li'

        class item(ItemElement):
            klass = BaseCalendarEvent

            obj_url = Link('./a[1]')
            obj_id = Regexp(Link('./a[1]'), r'\?(\d+)')
            obj_summary = CleanText('./a[1]')
            obj_start_date = Date(CleanText('./span[1]'))
            obj_category = CATEGORIES.CONCERT
            obj_status = STATUS.CONFIRMED



# -*- coding: utf-8 -*-

# Copyright(C) 2016      Vincent A
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import re

from weboob.capabilities.base import StringField
from weboob.capabilities.gallery import BaseGallery, BaseImage, CapGallery
from weboob.capabilities.image import CapImage, Thumbnail
from weboob.capabilities.paste import BasePaste, CapPaste
from weboob.tools.backend import Module
from weboob.tools.capabilities.paste import image_mime
from weboob.tools.date import datetime

from .browser import ImgurBrowser

__all__ = ['ImgurModule']


class ImgPaste(BasePaste):
    delete_url = StringField('URL to delete the image')

    @classmethod
    def id2url(cls, id):
        return 'https://imgur.com/%s' % id

    @property
    def raw_url(self):
        # TODO get the right extension
        return 'https://i.imgur.com/%s.png' % self.id


class Img(BaseImage):
    @property
    def thumbnail_url(self):
        return ImgPaste(self.id + 't').raw_url

    @property
    def raw_url(self):
        return 'https://i.imgur.com/%s.%s' % (self.id, self.ext)


class ImgGallery(BaseGallery):
    def __init__(self, *args, **kwargs):
        super(ImgGallery, self).__init__(*args, **kwargs)
        self._imgs = []

    @classmethod
    def id2url(cls, id):
        return 'https://imgur.com/gallery/%s' % id

    def iter_image(self):
        return self._imgs


class ImgurModule(Module, CapPaste, CapGallery, CapImage):
    NAME = 'imgur'
    DESCRIPTION = u'imgur image upload service'
    MAINTAINER = u'Vincent A'
    EMAIL = 'dev@indigo.re'
    LICENSE = 'AGPLv3+'
    VERSION = '1.3'

    BROWSER = ImgurBrowser

    IMGURL = re.compile(r'https?://(?:[a-z]+\.)?imgur.com/([a-zA-Z0-9]+)(?:\.[a-z]+)?$')
    GALLURL = re.compile(r'https?://(?:[a-z]+\.)?imgur.com/a/([a-zA-Z0-9]+)/?$')
    ID = re.compile(r'[0-9a-zA-Z]+$')

    # CapPaste
    def new_paste(self, *a, **kw):
        return ImgPaste(*a, **kw)

    def can_post(self, contents, title=None, public=None, max_age=None):
        if public is False:
            return 0
        elif re.search(r'[^a-zA-Z0-9=+/\s]', contents):
            return 0
        elif max_age:
            return 0
        else:
            mime = image_mime(contents, ('gif', 'jpeg', 'png', 'tiff', 'xcf', 'pdf'))
            return 20 * int(mime is not None)

    def get_paste(self, id):
        mtc = self.IMGURL.match(id)
        if mtc:
            id = mtc.group(1)
        elif not self.ID.match(id):
            return None

        paste = ImgPaste(id)
        bin = self.browser.open_raw(paste.raw_url).content
        paste.contents = bin.encode('base64')
        return paste

    def post_paste(self, paste, max_age=None):
        res = self.browser.post_image(b64=paste.contents, title=paste.title)
        paste.id = res['id']
        paste.delete_url = res['delete_url']
        return paste

    # CapGallery
    def _build_img(self, d, n, gallery):
        img = Img(d['id'], gallery=gallery, url=d['link'], index=n)
        img.title = d['title'] or u''
        img.description = d['description'] or u''
        img.ext = img.url.rsplit('.', 1)[-1]
        img.date = datetime.fromtimestamp(d['datetime'])
        img.thumbnail = Thumbnail(img.thumbnail_url)
        img.thumbnail.date = img.date
        img.nsfw = bool(d['nsfw'])
        img.size = d['size']
        return img

    def _build_gallery(self, d):
        gallery = ImgGallery(d['id'], url=d['link'])
        gallery.title = d['title'] or u''
        gallery.description = d['description'] or u''
        gallery.date = datetime.fromtimestamp(d['datetime'])
        gallery.thumbnail = Thumbnail(Img(d['cover']).thumbnail_url)

        if 'images' in d:
            for n, d in enumerate(d['images']):
                img = self._build_img(d, n, gallery)
                gallery._imgs.append(img)
            gallery.cardinality = len(gallery._imgs)

        return gallery

    def get_gallery(self, id):
        mtc = self.GALLURL.match(id)
        if mtc:
            id = mtc.group(1)
        elif not self.ID.match(id):
            return None

        d = self.browser.get_gallery(id)
        if d is None:
            return None
        return self._build_gallery(d)

    def iter_gallery_images(self, gallery):
        if not len(gallery._imgs):
            new = self.get_gallery(gallery.id)
            gallery._imgs = new._imgs
        return gallery._imgs

    def search_galleries(self, pattern, sortby=CapGallery.SEARCH_RELEVANCE):
        d = self.browser.search_items(pattern, sortby)
        for sub in d:
            if sub['is_album']:
                yield self._build_gallery(sub)

    # CapImage
    def get_file(self, id):
        mtc = self.IMGURL.match(id)
        if mtc:
            id = mtc.group(1)
        elif not self.ID.match(id):
            return None

        d = self.browser.get_image(id)
        if d is None:
            return None
        return self._build_img(d, 0, None)

    def search_file(self, pattern, sortby=CapGallery.SEARCH_RELEVANCE):
        d = self.browser.search_items(pattern, sortby)
        for sub in d:
            if not sub['is_album']:
                yield self._build_img(sub, 0, None)

    def search_image(self, pattern, sortby=CapGallery.SEARCH_RELEVANCE, nsfw=False):
        for img in self.search_file(pattern, sortby):
            if nsfw or not img.nsfw:
                yield img

    def fill_img(self, img, fields):
        if 'data' in fields:
            self.browser.fill_file(img, fields)
        if 'thumbnail' in fields and img.thumbnail:
            self.fillobj(img.thumbnail, None)
        return img

    OBJECTS = {Img: fill_img, Thumbnail: fill_img, BaseGallery: fill_img}



# -*- coding: utf-8 -*-

# Copyright(C) 2011  Clément Schreiner
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime

from weboob.tools.test import BackendTest


class MediawikiTest(BackendTest):
    MODULE = 'mediawiki'

    def test_get_content(self):
        obj = self.backend.get_content(u"Project:Sandbox")
        assert len(obj.content) > 0

    def test_iter_revisions(self):
        for rev in zip(range(10), self.backend.iter_revisions(u"Project:Sandbox")):
            pass

    def test_push_content(self):
        content = self.backend.get_content(u"Project:Sandbox")
        content.content = "%s\nhello %s" % (content.content, datetime.now())
        # ^ warning: wikipedia seems to have blocked lines starting with "test"...
        self.backend.push_content(content, message="test weboob", minor=True)
        new_content = self.backend.get_content(u"Project:Sandbox")
        self.assertEquals(content.content, new_content.content)

    def test_content_preview(self):
        content = self.backend.get_content(u"Project:Sandbox")
        self.backend.get_content_preview(content)

    def test_search_image(self):
        it = iter(self.backend.search_file('logo'))
        for _, img in zip(range(3), it):
            assert img
            assert img.title
            assert img.ext
            assert img.page_url
            assert img.size
            if not img.url:
                img = self.backend.fillobj(img, ['url'])
            assert img.url
            assert img.thumbnail.url



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Christophe Benz
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.capabilities.messages import CantSendMessage, CapMessages, CapMessagesPost
from weboob.capabilities.account import CapAccount, StatusField
from weboob.tools.backend import Module, BackendConfig
from weboob.tools.value import Value, ValueBackendPassword

from .browser import SfrBrowser


__all__ = ['SfrModule']


class SfrModule(Module, CapAccount, CapMessages, CapMessagesPost):
    NAME = 'sfr'
    MAINTAINER = u'Christophe Benz'
    EMAIL = 'christophe.benz@gmail.com'
    VERSION = '1.3'
    DESCRIPTION = 'SFR French mobile phone provider'
    LICENSE = 'AGPLv3+'
    CONFIG = BackendConfig(Value('login', label='Login'),
                           ValueBackendPassword('password', label='Password'))
    BROWSER = SfrBrowser
    ACCOUNT_REGISTER_PROPERTIES = None

    def create_default_browser(self):
        return self.create_browser(self.config['login'].get(), self.config['password'].get())

    # CapMessagesPost methods

    def get_account_status(self):
        with self.browser:
            return (StatusField('nb_remaining_free_sms', 'Number of remaining free SMS',
                                self.browser.get_nb_remaining_free_sms()),)

    def post_message(self, message):
        if not message.content.strip():
            raise CantSendMessage(u'Message content is empty.')
        with self.browser:
            self.browser.post_message(message)



# -*- coding: utf-8 -*-

# Copyright(C) 2013 Mathieu Jourdan
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.capabilities.bill import CapDocument, SubscriptionNotFound,\
    DocumentNotFound, Subscription, Bill
from weboob.tools.backend import Module, BackendConfig
from weboob.tools.value import ValueBackendPassword
from .browser import GdfSuez

__all__ = ['GdfSuezModule']


class GdfSuezModule(Module, CapDocument):
    NAME = 'gdfsuez'
    MAINTAINER = u'Mathieu Jourdan'
    EMAIL = 'mathieu.jourdan@gresille.org'
    VERSION = '1.3'
    LICENSE = 'AGPLv3+'
    DESCRIPTION = u'GDF-Suez French energy provider'
    CONFIG = BackendConfig(ValueBackendPassword('login',
                                                label='Account ID (e-mail)',
                                                masked=False),
                           ValueBackendPassword('password',
                                                label='Password',
                                                masked=True)
                           )
    BROWSER = GdfSuez

    def create_default_browser(self):
        return self.create_browser(self.config['login'].get(),
                                   self.config['password'].get())

    def iter_subscription(self):
        for subscription in self.browser.get_subscription_list():
            yield subscription

    def get_subscription(self, _id):
        if not _id.isdigit():
            raise SubscriptionNotFound()
        with self.browser:
            subscription = self.browser.get_subscription(_id)
        if not subscription:
            raise SubscriptionNotFound()
        else:
            return subscription

    def iter_documents_history(self, subscription):
        if not isinstance(subscription, Subscription):
            subscription = self.get_subscription(subscription)
        with self.browser:
            for history in self.browser.get_history(subscription):
                yield history

    def get_details(self, subscription):
        if not isinstance(subscription, Subscription):
            subscription = self.get_subscription(subscription)
        with self.browser:
            for detail in self.browser.get_details(subscription):
                yield detail

    def iter_documents(self, subscription):
        if not isinstance(subscription, Subscription):
            subscription = self.get_subscription(subscription)
        with self.browser:
            for bill in self.browser.iter_documents():
                yield bill

    def get_document(self, id):
        with self.browser:
            bill = self.browser.get_document(id)
        if not bill:
            raise DocumentNotFound()
        else:
            return bill

    def download_document(self, bill):
        if not isinstance(bill, Bill):
            bill = self.get_document(bill)
        with self.browser:
            return self.browser.readurl(bill.url)



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Romain Bignon
# Copyright(C) 2012 François Revol
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.deprecated.browser import Browser
from weboob.deprecated.browser.decorators import id2url

#from .pages.index import IndexPage
from .pages import VideoPage
from .video import EuroparlVideo


__all__ = ['EuroparlBrowser']


class EuroparlBrowser(Browser):
    DOMAIN = 'europarl.europa.eu'
    ENCODING = None
    PAGES = {r'http://[w\.]*europarl\.europa\.eu/ep-live/(?P<lang>\w+)/committees/video\?.*event=(?P<id>[^&]+).*': VideoPage,
             r'http://[w\.]*europarl\.europa\.eu/ep-live/(?P<lang>\w+)/other-events/video\?.*event=(?P<id>[^&]+).*': VideoPage
#TODO:plenaries
#            r'http://[w\.]*europarl\.europa\.eu/ep-live/(?P<lang>\w+)/plenary/video\?.*date=(?P<id>[^&]+).*': VideoPage
#            r'http://[w\.]*europarl\.europa\.eu/ep-live/(?P<lang>\w+)/plenary/video\?.*debate=(?P<id>[^&]+).*': VideoPage
            }

    @id2url(EuroparlVideo.id2url)
    def get_video(self, url, video=None):
        self.location(url)
        return self.page.get_video(video)

    # def search_videos(self, pattern, sortby):
    #     return None
    #     self.location(self.buildurl('http://europarltv.europa.eu/en/search%s' % sortby, query=pattern.encode('utf-8')))
    #     assert self.is_on_page(IndexPage)
    #     return self.page.iter_videos()

    # def latest_videos(self):
    #     self.home()
    #     assert self.is_on_page(IndexPage)
    #     return self.page.iter_videos()



# -*- coding: utf-8 -*-

# Copyright(C) 2014      Bezleputh
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from collections import OrderedDict

from weboob.tools.backend import Module, BackendConfig
from weboob.capabilities.calendar import CapCalendarEvent, CATEGORIES
from weboob.tools.value import Value

from .browser import AgendadulibreBrowser


__all__ = ['AgendadulibreModule']


class AgendadulibreModule(Module, CapCalendarEvent):
    NAME = 'agendadulibre'
    DESCRIPTION = u'agendadulibre website'
    MAINTAINER = u'Bezleputh'
    EMAIL = 'carton_ben@yahoo.fr'
    LICENSE = 'AGPLv3+'
    VERSION = '1.3'
    ASSOCIATED_CATEGORIES = [CATEGORIES.CONF]
    BROWSER = AgendadulibreBrowser

    region_choices = OrderedDict([(k, u'%s (%s)' % (v, k)) for k, v in sorted({
        "https://www.agendadulibre.org": u'--France--',
        "https://www.agendadulibre.org#3": u'Auvergne-Rhône-Alpes',
        "https://www.agendadulibre.org#5": u'Bourgogne-Franche-Comté',
        "https://www.agendadulibre.org#6": u'Bretagne',
        "https://www.agendadulibre.org#7": u'Centre-Val de Loire',
        "https://www.agendadulibre.org#30": u'Collectivité sui generis',
        "https://www.agendadulibre.org#29": u'Collectivités d\'outre-mer',
        "https://www.agendadulibre.org#9": u'Corse',
        "https://www.agendadulibre.org#1": u'Grand Est',
        "https://www.agendadulibre.org#23": u'Guadeloupe',
        "https://www.agendadulibre.org#24": u'Guyane',
        "https://www.agendadulibre.org#17": u'Hauts-de-France',
        "https://www.agendadulibre.org#12": u'Île-de-France',
        "https://www.agendadulibre.org#31": u'Internet',
        "https://www.agendadulibre.org#26": u'La Réunion',
        "https://www.agendadulibre.org#25": u'Martinique',
        "https://www.agendadulibre.org#28": u'Mayotte',
        "https://www.agendadulibre.org#4": u'Normandie',
        "https://www.agendadulibre.org#2": u'Nouvelle-Aquitaine',
        "https://www.agendadulibre.org#13": u'Occitanie',
        "https://www.agendadulibre.org#18": u'Pays de la Loire',
        "https://www.agendadulibre.org#21": u'Provence-Alpes-Côte d\'Azur',
        "https://www.agendadulibre.be": u'--Belgique--',
        "https://www.agendadulibre.be#11": u'Antwerpen',
        "https://www.agendadulibre.be#10": u'Brabant wallon',
        "https://www.agendadulibre.be#9": u'Bruxelles-Capitale',
        "https://www.agendadulibre.be#8": u'Hainaut',
        "https://www.agendadulibre.be#7": u'Liege',
        "https://www.agendadulibre.be#6": u'Limburg',
        "https://www.agendadulibre.be#5": u'Luxembourg',
        "https://www.agendadulibre.be#4": u'Namur',
        "https://www.agendadulibre.be#3": u'Oost-Vlaanderen',
        "https://www.agendadulibre.be#2": u'Vlaams-Brabant',
        "https://www.agendadulibre.be#1": u'West-Vlaanderen',
        "https://www.agendadulibre.ch": u'--Suisse--',
        "https://www.agendadulibre.ch#15": u'Appenzell Rhodes-Extérieures',
        "https://www.agendadulibre.ch#16": u'Appenzell Rhodes-Intérieures',
        "https://www.agendadulibre.ch#19": u'Argovie',
        "https://www.agendadulibre.ch#13": u'Bâle-Campagne',
        "https://www.agendadulibre.ch#12": u'Bâle-Ville',
        "https://www.agendadulibre.ch#2": u'Berne',
        "https://www.agendadulibre.ch#10": u'Fribourg',
        "https://www.agendadulibre.ch#25": u'Genève',
        "https://www.agendadulibre.ch#8": u'Glaris',
        "https://www.agendadulibre.ch#18": u'Grisons',
        "https://www.agendadulibre.ch#26": u'Jura',
        "https://www.agendadulibre.ch#3": u'Lucerne',
        "https://www.agendadulibre.ch#24": u'Neuchâtel',
        "https://www.agendadulibre.ch#7": u'Nidwald',
        "https://www.agendadulibre.ch#6": u'Obwald',
        "https://www.agendadulibre.ch#17": u'Saint-Gall',
        "https://www.agendadulibre.ch#14": u'Schaffhouse',
        "https://www.agendadulibre.ch#5": u'Schwytz',
        "https://www.agendadulibre.ch#11": u'Soleure',
        "https://www.agendadulibre.ch#21": u'Tessin',
        "https://www.agendadulibre.ch#20": u'Thurgovie',
        "https://www.agendadulibre.ch#4": u'Uri',
        "https://www.agendadulibre.ch#23": u'Valais',
        "https://www.agendadulibre.ch#22": u'Vaud',
        "https://www.agendadulibre.ch#9": u'Zoug',
        "https://www.agendadulibre.ch#1": u'Zurich',
    }.items())])

    CONFIG = BackendConfig(Value('region', label=u'Region', choices=region_choices))

    def create_default_browser(self):
        choice = self.config['region'].get().split('#')
        selected_region = '' if len(choice) < 2 else choice[-1]
        return self.create_browser(website=choice[0], region=selected_region)

    def search_events(self, query):
        return self.browser.list_events(query.start_date,
                                        query.end_date,
                                        query.city,
                                        query.categories)

    def list_events(self, date_from, date_to=None):
        return self.browser.list_events(date_from, date_to)

    def get_event(self, event_id):
        return self.browser.get_event(event_id)

    def fill_obj(self, event, fields):
        event = self.browser.get_event(event.id, event)
        choice = self.config['region'].get().split('#')
        selected_region = '' if len(choice) < 2 else choice[-1]
        if selected_region == '23':
            event.timezone = 'America/Guadeloupe'
        elif selected_region == '24':
            event.timezone = 'America/Guyana'
        elif selected_region == '26':
            event.timezone = 'Indian/Reunion'
        elif selected_region == '25':
            event.timezone = 'America/Martinique'
        else:
            event.timezone = 'Europe/Paris'
        return event

    OBJECTS = {AgendadulibreBrowser: fill_obj}



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Nicolas Duhamel
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from datetime import datetime

from weboob.capabilities.bank import (
    TransferError, TransferBankError, Transfer, TransferStep, NotAvailable, Recipient, AccountNotFound, AddRecipientError
)
from weboob.capabilities.base import find_object
from weboob.browser.pages import LoggedPage
from weboob.browser.filters.standard import CleanText, Env, Regexp, Date, CleanDecimal
from weboob.browser.filters.html import Attr, Link
from weboob.browser.elements import ListElement, ItemElement, method, SkipItem
from weboob.tools.capabilities.bank.transactions import FrenchTransaction
from weboob.tools.capabilities.bank.iban import is_iban_valid
from weboob.tools.value import Value

from .base import MyHTMLPage

class CheckTransferError(MyHTMLPage):
    def on_load(self):
        MyHTMLPage.on_load(self)
        error = CleanText(u'//span[@class="app_erreur"] | //p[@class="warning"] | //p[contains(text(), "Votre virement n\'a pas pu être enregistré")]')(self.doc)
        if error and not u'Votre demande de virement a été enregistrée le' in error:
            raise TransferBankError(message=error)


class TransferChooseAccounts(LoggedPage, MyHTMLPage):
    def is_inner(self, text):
        for option in self.doc.xpath('//select[@id="donneesSaisie.idxCompteEmetteur"]/option'):
            if text == CleanText('.')(option):
                return True
        return False

    @method
    class iter_recipients(ListElement):
        def condition(self):
            return any(self.env['account_id'] in CleanText('.')(option) for option in self.page.doc.xpath('//select[@id="donneesSaisie.idxCompteEmetteur"]/option'))

        # You're not dreaming, this is real life
        item_xpath = '//select[@id="caca"]/option'

        class Item(ItemElement):
            klass = Recipient

            def condition(self):
                return self.el.attrib['value'] != '-1'

            def validate(self, obj):
                # Some international external recipients show those infos:
                # INT - CA - 0815304220511006                   - CEGEP CANADA
                # Skipping those for the moment.
                return not obj.iban or is_iban_valid(obj.iban)

            obj_category = Env('category')
            obj_label = Env('label')
            obj_id = Env('id')
            obj_currency = u'EUR'
            obj_iban = Env('iban')
            obj_bank_name = Env('bank_name')
            obj__value = Attr('.', 'value')

            def obj_enabled_at(self):
                return datetime.now().replace(microsecond=0)

            def parse(self, el):
                if any(s in CleanText('.')(el) for s in ['Avoir disponible', 'Solde']) or self.page.is_inner(CleanText('.')(el)):
                    self.env['category'] = u'Interne'
                else:
                    self.env['category'] = u'Externe'
                if self.env['category'] == u'Interne':
                    _id = Regexp(CleanText('.'), '- (.*?) -')(el)
                    if _id == self.env['account_id']:
                        raise SkipItem()
                    try:
                        account = find_object(self.page.browser.get_accounts_list(), id=_id, error=AccountNotFound)
                        self.env['id'] = _id
                        self.env['label'] = account.label
                        self.env['iban'] = account.iban
                    except AccountNotFound:
                        self.env['id'] = Regexp(CleanText('.'), '- (.*?) -')(el).replace(' ', '')
                        self.env['iban'] = NotAvailable
                        label = CleanText('.')(el).split('-')
                        holder = label[-1] if not any(string in label[-1] for string in ['Avoir disponible', 'Solde']) else label[-2]
                        self.env['label'] = '%s %s' % (label[0].strip(), holder.strip())
                    self.env['bank_name'] = u'La Banque Postale'

                else:
                    self.env['id'] = self.env['iban'] = Regexp(CleanText('.'), '- (.*?) -')(el).replace(' ', '')
                    self.env['label'] = Regexp(CleanText('.'), '- (.*?) - (.*)', template='\\2')(el).strip()
                    first_part = CleanText('.')(el).split('-')[0].strip()
                    self.env['bank_name'] = u'La Banque Postale' if first_part in ['CCP', 'PEL'] else NotAvailable

                if self.env['id'] in self.parent.objects: # user add two recipients with same iban...
                    raise SkipItem()

    def init_transfer(self, account_id, recipient_value):
        matched_values = [Attr('.', 'value')(option) for option in self.doc.xpath('//select[@id="donneesSaisie.idxCompteEmetteur"]/option') \
                          if account_id in CleanText('.')(option)]
        assert len(matched_values) == 1
        form = self.get_form(xpath='//form[@class="formvirement"]')
        form['donneesSaisie.idxCompteReceveur'] = recipient_value
        form['donneesSaisie.idxCompteEmetteur'] = matched_values[0]
        form.submit()


class CompleteTransfer(LoggedPage, CheckTransferError):
    def complete_transfer(self, amount, transfer):
        form = self.get_form(xpath='//form[@method]')
        form['montant'] = amount
        if 'commentaire' in form and transfer.label:
            form['commentaire'] = transfer.label
        form['dateVirement'] = transfer.exec_date.strftime('%d/%m/%Y')
        form.submit()


class TransferConfirm(LoggedPage, CheckTransferError):
    def is_here(self):
        return not CleanText('//p[contains(text(), "Vous pouvez le consulter dans le menu")]')(self.doc)

    def double_auth(self, transfer):
        code_needed = CleanText('//label[@for="code_securite"]')(self.doc)
        if code_needed:
            raise TransferStep(transfer, Value('code', label= code_needed))

    def confirm(self):
        form = self.get_form(id='formID')
        form.submit()

    def handle_response(self, account, recipient, amount, reason):
        account_txt = CleanText('//form//dl/dt[span[contains(text(), "biter")]]/following::dd[1]', replace=[(' ', '')])(self.doc)
        recipient_txt = CleanText('//form//dl/dt[span[contains(text(), "diter")]]/following::dd[1]', replace=[(' ', '')])(self.doc)
        try:
            assert account.id in account_txt or ''.join(account.label.split()) == account_txt
            assert recipient.id in recipient_txt or ''.join(recipient.label.split()) == recipient_txt
        except AssertionError:
            raise TransferError('Something went wrong')
        r_amount =  CleanDecimal('//form//dl/dt[span[contains(text(), "Montant")]]/following::dd[1]', replace_dots=True)(self.doc)
        exec_date = Date(CleanText('//form//dl/dt[span[contains(text(), "Date")]]/following::dd[1]'), dayfirst=True)(self.doc)
        currency = FrenchTransaction.Currency('//form//dl/dt[span[contains(text(), "Montant")]]/following::dd[1]')(self.doc)

        transfer = Transfer()
        transfer.currency = currency
        transfer.amount = r_amount
        transfer.account_iban = account.iban
        transfer.recipient_iban = recipient.iban
        transfer.account_id = account.id
        transfer.recipient_id = recipient.id
        transfer.exec_date = exec_date
        transfer.label = reason
        transfer.account_label = account.label
        transfer.recipient_label = recipient.label
        transfer.account_balance = account.balance
        return transfer


class TransferSummary(LoggedPage, CheckTransferError):
    def handle_response(self, transfer):
        # NotAvailable in case of future exec_date not on a working day.
        transfer.id = Regexp(CleanText('//div[@class="bloc Tmargin"]'), 'virement N.+ (\d+) ', default=NotAvailable)(self.doc)
        if not transfer.id:
            transfer.exec_date = Date(Regexp(CleanText('//div[@class="bloc Tmargin"]'), 'suivant \(([\d\/]+)\)'), dayfirst=True)(self.doc)
        return transfer


class CreateRecipient(LoggedPage, MyHTMLPage):
    def choose_country(self, recipient, is_bp_account):
        # if this is present, we can't add recipient currently
        more_security_needed = self.doc.xpath(u'//iframe[@title="Sécurité renforcée non adhérents"]')
        if more_security_needed:
            raise AddRecipientError(u"Pour activer le service Certicode, nous vous invitons à vous rapprocher de votre Conseiller en Bureau de Poste.")

        form = self.get_form(name='SaisiePaysBeneficiaireVirement')
        form['compteLBP'] = str(is_bp_account).lower()
        form['beneficiaireBean.paysDestination'] = recipient.iban[:2]
        form.submit()


class ValidateCountry(LoggedPage, MyHTMLPage):
    def populate(self, recipient):
        form = self.get_form(name='CaracteristiquesBeneficiaireVirement')
        form['beneficiaireBean.nom'] = recipient.label
        form['beneficiaireBean.ibans[1].valeur'] = recipient.iban[2:4]
        form['beneficiaireBean.ibans[2].valeur'] = recipient.iban[4:8]
        form['beneficiaireBean.ibans[3].valeur'] = recipient.iban[8:12]
        form['beneficiaireBean.ibans[4].valeur'] = recipient.iban[12:16]
        form['beneficiaireBean.ibans[5].valeur'] = recipient.iban[16:20]
        form['beneficiaireBean.ibans[6].valeur'] = recipient.iban[20:24]
        form['beneficiaireBean.ibans[7].valeur'] = recipient.iban[24:]
        form['beneficiaireBean.intituleCompte'] = recipient.label
        form.submit()


class ValidateRecipient(LoggedPage, MyHTMLPage):
    def is_bp_account(self):
        msg = CleanText('//span[has-class("app_erreur")]')(self.doc)
        return msg == u'Le n° de compte que vous avez saisi appartient à La Banque Postale, veuillez vérifier votre saisie.'

    def get_confirm_link(self):
        return Link('//a[@title="confirmer la creation"]')(self.doc)


class ConfirmPage(LoggedPage, MyHTMLPage):
    def set_browser_form(self):
        form = self.get_form(name='SaisieOTP')
        self.browser.recipient_form = dict((k, v) for k, v in form.items() if v)
        self.browser.recipient_form['url'] = form.url


class RcptSummary(LoggedPage, MyHTMLPage):
    pass



# -*- coding: utf-8 -*-

# Copyright(C) 2013 Julien Veyssier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.browser.pages import HTMLPage, pagination
from weboob.browser.elements import ItemElement, ListElement, method
from weboob.browser.filters.standard import Regexp, CleanText, Format, Env, BrowserURL, CleanDecimal, Eval
from weboob.browser.filters.json import Dict
from weboob.capabilities.recipe import Recipe, Comment
from weboob.tools.json import json
import re


class ResultsPage(HTMLPage):
    """ Page which contains results as a list of recipies
    """
    @pagination
    @method
    class iter_recipes(ListElement):
        item_xpath = '//div[@class="m_resultats_liste_recherche"]/div[has-class("recette_classique")]'

        def next_page(self):
            str_results = Regexp(CleanText('//div[@class="m_resultats_recherche_titre"]'),
                                 '.* - (\d* / \d*) .*')(self)
            results = str_results.split('/')
            if int(results[0]) - int(results[1]) < 10:
                return BrowserURL('search', pattern=Env('pattern'), start=int(results[0]))(self)

        class item(ItemElement):
            klass = Recipe
            obj_id = Regexp(CleanText('./div/div[@class="m_titre_resultat"]/a/@href'),
                            '/recettes/recette_(.*).aspx')
            obj_title = CleanText('./div/div[@class="m_titre_resultat"]/a')
            obj_short_description = Format('%s. %s',
                                           CleanText('./div/div[@class="m_detail_recette"]'),
                                           CleanText('./div/div[@class="m_texte_resultat"]'))


class RecipePage(HTMLPage):
    """ Page which contains a recipe
    """

    @method
    class get_recipe(ItemElement):
        klass = Recipe

        def parse(self, el):
            json_content = CleanText(u'//script[@type="application/ld+json"]',
                                     replace=[('//<![CDATA[ ', ''),
                                              (' //]]>', '')])(self)
            self.el = json.loads(json_content)

        obj_id = Env('id')
        obj_title = Dict('name')
        obj_ingredients = Dict('recipeIngredient')

        obj_thumbnail_url = Dict('image')
        obj_picture_url = Dict('image')

        def obj_instructions(self):
            str = Dict('recipeInstructions')(self)
            return re.sub(r'(\d+\.)', r'\n\1', str)

        obj_preparation_time = Eval(int, CleanDecimal(Dict('prepTime')))
        obj_cooking_time = Eval(int, CleanDecimal(Dict('cookTime')))

        def obj_nb_person(self):
            return [Dict('recipeYield')(self)]


class CommentsPage(HTMLPage):
    """ Page which contains a comments
    """

    @method
    class get_comments(ListElement):
        item_xpath = '//div[@class="commentaire"]/div/table/tr'
        ignore_duplicate = True

        class item(ItemElement):
            klass = Comment

            obj_author = CleanText('./td/div[@class="txtCommentaire"]/div[1]')
            obj_rate = CleanText('./td/div[@class="bulle"]')

            def obj_text(self):
                return CleanText('./td/div[@class="txtCommentaire"]')(self)

            obj_id = CleanText('./td/div[@class="txtCommentaire"]/div[1]')



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2016  weboob project
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

from collections import deque
from weakref import WeakKeyDictionary

from PyQt5.QtCore import Qt, QObject, QAbstractItemModel, QModelIndex, \
                         QSortFilterProxyModel, QVariant, pyqtSignal as Signal,\
                         pyqtSlot as Slot
from PyQt5.QtGui import QIcon, QImage, QPixmap, QPixmapCache, \
                        QStandardItemModel, QStandardItem
from PyQt5.QtWidgets import QApplication

from weboob.capabilities.base import NotAvailable, NotLoaded
from weboob.capabilities.collection import BaseCollection
from weboob.capabilities.file import BaseFile
from weboob.capabilities.gallery import BaseGallery, BaseImage as GBaseImage
from weboob.capabilities.gauge import Gauge, GaugeSensor
# TODO expand other cap objects when needed

from .qt import QtDo


__all__ = ['BackendListModel', 'ResultModel', 'FilterTypeModel']


class BackendListModel(QStandardItemModel):
    """Model for displaying a backends list with icons"""

    RoleBackendName = Qt.UserRole
    RoleCapability = Qt.UserRole + 1

    def __init__(self, weboob, *args, **kwargs):
        super(BackendListModel, self).__init__(*args, **kwargs)
        self.weboob = weboob

    def addBackends(self, cap=None, entry_all=True, entry_title=False):
        """
        Populate the model by adding backends.

        Appends backends to the model, without clearing previous entries.
        For each entry in the model, the cap name is stored under role
        RoleBackendName and the capability object under role
        RoleCapability.

        :param cap: capabilities to add (None to add all loaded caps)
        :param entry_all: if True, add a "All backends" entry
        :param entry_title: if True, add a disabled entry with the cap name
        """

        if entry_title:
            if cap:
                capname = cap.__name__
            else:
                capname = '(All capabilities)'

            item = QStandardItem(capname)
            item.setEnabled(False)
            self.appendRow(item)

        first = True
        for backend in self.weboob.iter_backends(caps=cap):
            if first and entry_all:
                item = QStandardItem('(All backends)')
                item.setData('', self.RoleBackendName)
                item.setData(cap, self.RoleCapability)
                self.appendRow(item)
            first = False

            item = QStandardItem(backend.name)
            item.setData(backend.name, self.RoleBackendName)
            item.setData(cap, self.RoleCapability)
            minfo = self.weboob.repositories.get_module_info(backend.NAME)
            icon_path = self.weboob.repositories.get_module_icon_path(minfo)
            if icon_path:
                pixmap = QPixmapCache.find(icon_path)
                if not pixmap:
                    pixmap = QPixmap(QImage(icon_path))
                item.setIcon(QIcon(pixmap))
            self.appendRow(item)


class DoWrapper(QtDo):
    """Wrapper for QtDo to use in DoQueue."""

    def __init__(self, *args, **kwargs):
        super(DoWrapper, self).__init__(*args, **kwargs)
        self.do_args = None

    def do(self, *args, **kwargs):
        self.do_args = (args, kwargs)

    def start(self):
        super(DoWrapper, self).do(*self.do_args[0], **self.do_args[1])
        self.do_args = None


class DoQueue(QObject):
    """Queue to limit the number of parallel Do processes."""

    def __init__(self):
        super(DoQueue, self).__init__()
        self.max_tasks = 10
        self.running = set()
        self.queue = deque()

    def add(self, doer):
        doer.finished.connect(self._finished)

        if len(self.running) < self.max_tasks:
            self.running.add(doer)
            doer.start()
        else:
            self.queue.append(doer)

    @Slot()
    def _finished(self):
        try:
            self.running.remove(self.sender())
        except KeyError:
            return

        try:
            doer = self.queue.popleft()
        except IndexError:
            return
        self.running.add(doer)
        doer.start()

    def stop(self):
        doers = list(self.running) + list(self.queue)
        self.running, self.queue = set(), deque()
        for do in doers:
            do.stop()


class ByIdDict(dict):
    """dict keeping objects by their `id()`

    This is used to store BaseObjects as key, because they are unhashable and
    so can't be put in a regular dict else.
    """

    def __init__(self):
        super(ByIdDict, self).__init__()
        self.objs = {}

    def __getitem__(self, k):
        return super(ByIdDict, self).__getitem__(id(k))

    def get(self, k, default=None):
        return super(ByIdDict, self).get(id(k), default)

    def __contains__(self, k):
        return super(ByIdDict, self).__contains__(id(k))

    def __setitem__(self, k, v):
        super(ByIdDict, self).__setitem__(id(k), v)
        self.objs[id(k)] = k

    def setdefault(self, k, v):
        self.objs[id(k)] = k
        return super(ByIdDict, self).setdefault(id(k), v)

    def __delitem__(self, k):
        super(ByIdDict, self).__delitem__(id(k))
        del self.objs[id(k)]

    def pop(self, k, *args, **kwargs):
        self.objs.pop(id(k), None)
        return super(ByIdDict, self).pop(id(k), *args, **kwargs)

    def clear(self):
        super(ByIdDict, self).clear()
        self.objs.clear()


class ResultModel(QAbstractItemModel):
    """Model for displaying objects and collections"""

    RoleObject = Qt.UserRole
    RoleCapability = Qt.UserRole + 1
    RoleBackendName = Qt.UserRole + 2

    jobAdded = Signal()
    jobFinished = Signal()

    def __init__(self, weboob, *args, **kwargs):
        super(ResultModel, self).__init__(*args, **kwargs)
        self.weboob = weboob
        self.resource_classes = []
        self.children = ByIdDict()
        self.parents = ByIdDict()
        self.columns = []

        self.limit = None

        self.jobs = DoQueue()
        self.jobExpanders = WeakKeyDictionary()
        self.jobFillers = WeakKeyDictionary()

    def __del__(self):
        try:
            self.jobs.stop()
        except:
            pass

    # configuration/general operation
    def setLimit(self, limit):
        self.limit = limit

    def clear(self):
        """Empty the model completely"""
        self.jobs.stop()
        #n = len(self.children.get(None, []))
        self.beginResetModel()
        #if n:
        #    self.beginRemoveRows(QModelIndex(), 0, max(0, n - 1))
        self.children.clear()
        self.parents.clear()
        self.endResetModel()
        #if n:
        #    self.endRemoveRows()

    @Slot(object)
    def _gotRootDone(self, obj):
        self._addToRoot(obj)

    def addRootDo(self, *args, **kwargs):
        """Make a weboob.do and add returned items to root of model"""
        process = DoWrapper(self.weboob, None)
        process.gotResponse.connect(self._gotRootDone)
        process.finished.connect(self.jobFinished)

        process.do(*args, **kwargs)
        self.jobAdded.emit()
        self.jobs.add(process)

    def addRootDoLimit(self, cls, *args, **kwargs):
        app = QApplication.instance()
        if cls is None:
            fields = None
        else:
            fields = self._expandableFields(cls)
        return self.addRootDo(app._do_complete, self.limit, fields, *args, **kwargs)

    def addRootItems(self, objs):
        for obj in objs:
            self._addToRoot(obj)

    def setColumnFields(self, columns):
        self.columns = columns

    def setResourceClasses(self, classes):
        """Set accepted object classes for CapCollection.iter_resources"""
        self.resource_classes = classes

    # internal operation
    def _addToRoot(self, obj):
        self._addItem(obj, None, QModelIndex())

    def _addItem(self, obj, parent, parent_qidx):
        children = self.children.setdefault(parent, [])
        n = len(children)
        self.beginInsertRows(parent_qidx, n, n)
        children.append(obj)
        self.parents[obj] = parent
        self.endInsertRows()

    @Slot(object)
    def _expanderGotResponse(self, obj):
        parent, parent_qidx = self.jobExpanders[self.sender()]
        self._addItem(obj, parent, parent_qidx)

    def _prepareExpanderJob(self, parent, parent_qidx):
        process = DoWrapper(self.weboob, None)
        process.finished.connect(self.jobFinished)
        process.gotResponse.connect(self._expanderGotResponse)
        self.jobExpanders[process] = (parent, parent_qidx)
        return process

    def _expandableFields(self, cls):
        fields = set()

        for col in self.columns:
            for f in col:
                if f == 'id' or f in cls._fields:
                    fields.add(f)
        if 'thumbnail' in cls._fields:
            fields.add('thumbnail')

        return list(fields)

    def expandGauge(self, gauge, qidx):
        app = QApplication.instance()
        fields = self._expandableFields(GaugeSensor)

        process = self._prepareExpanderJob(gauge, qidx)
        process.do(app._do_complete, self.limit, fields, 'iter_sensors', gauge.id, backends=[gauge.backend])
        self.jobAdded.emit()
        self.jobs.add(process)

    def expandGallery(self, gall, qidx):
        app = QApplication.instance()
        fields = self._expandableFields(GBaseImage)

        process = self._prepareExpanderJob(gall, qidx)
        process.do(app._do_complete, self.limit, fields, 'iter_gallery_images', gall, backends=[gall.backend])
        self.jobAdded.emit()
        self.jobs.add(process)

    def expandCollection(self, coll, qidx):
        app = QApplication.instance()
        if len(self.resource_classes) == 1:
            fields = self._expandableFields(self.resource_classes[0])
        else:
            fields = None
            # at this point, we don't know the class of each object
            # FIXME reimplement _do_complete obj to filter dynamically

        process = self._prepareExpanderJob(coll, qidx)
        process.do(app._do_complete, self.limit, fields, 'iter_resources', self.resource_classes, coll.split_path, backends=[coll.backend])
        self.jobAdded.emit()
        self.jobs.add(process)

    def expandObj(self, obj, qidx):
        if isinstance(obj, BaseCollection):
            self.expandCollection(obj, qidx)
        elif isinstance(obj, BaseGallery):
            self.expandGallery(obj, qidx)
        elif isinstance(obj, Gauge):
            self.expandGauge(obj, qidx)

    def _getBackend(self, qidx):
        while qidx.isValid():
            obj = qidx.internalPointer()
            if obj.backend:
                return obj.backend
            qidx = qidx.parent()

    def fillObj(self, obj, fields, qidx):
        process = DoWrapper(self.weboob, None)
        self.jobFillers[process] = qidx
        process.gotResponse.connect(self._fillerGotResponse)
        process.finished.connect(self.jobFinished)

        process.do('fillobj', obj, fields, backends=qidx.data(self.RoleBackendName))
        self.jobAdded.emit()
        self.jobs.add(process)

    @Slot(object)
    def _fillerGotResponse(self, _):
        qidx = self.jobFillers[self.sender()]
        self.dataChanged.emit(qidx, qidx)

    # Qt model methods
    def index(self, row, col, parent_qidx):
        parent = parent_qidx.internalPointer()
        children = self.children.get(parent, ())
        if row >= len(children):
            return QModelIndex()
        return self.createIndex(row, col, children[row])

    def parent(self, qidx):
        obj = qidx.internalPointer()
        if obj is None:
            return QModelIndex()

        parent = self.parents.get(obj)
        if parent is None:
            return QModelIndex()

        gparent = self.parents[parent]
        row = self.children[gparent].index(parent)
        return self.createIndex(row, 0, parent)

    def flags(self, qidx):
        obj = qidx.internalPointer()
        if obj is None:
            return Qt.NoItemFlags
        else:
            return Qt.ItemIsSelectable | Qt.ItemIsEnabled

    def rowCount(self, qidx):
        if qidx.column() != 0 and qidx.isValid():
            return 0
        obj = qidx.internalPointer()
        return len(self.children.get(obj, []))

    def columnCount(self, qidx):
        if qidx.column() != 0 and qidx.isValid():
            return 0
        return len(self.columns)

    def data(self, qidx, role):
        obj = qidx.internalPointer()
        if obj is None:
            return QVariant()

        if role == self.RoleBackendName:
            return QVariant(self._getBackend(qidx))
        elif role == self.RoleObject:
            return QVariant(obj)
        elif role == Qt.DecorationRole:
            return self._dataIcon(obj, qidx)
        elif role == Qt.DisplayRole:
            return self._dataText(obj, qidx)
        return QVariant()

    def headerData(self, section, orientation, role):
        if role != Qt.DisplayRole:
            return QVariant()
        elif section >= len(self.columns):
            return QVariant()
        return '/'.join(self.columns[section])

    def hasChildren(self, qidx):
        obj = qidx.internalPointer()

        if isinstance(obj, BaseFile):
            return False
        # assume there are children, so a view may ask fetching
        children = self.children.get(obj, [True])
        return bool(len(children))

    def canFetchMore(self, qidx):
        obj = qidx.internalPointer()
        if obj is None:
            return False
        return obj not in self.children

    def fetchMore(self, qidx):
        if not self.canFetchMore(qidx):
            return

        obj = qidx.internalPointer()
        if obj is not None:
            self.children.setdefault(obj, [])
            self.expandObj(obj, qidx)

    # overridable
    def _dataText(self, obj, qidx):
        fields = self.columns[qidx.column()]
        for field in fields:
            if hasattr(obj, field):
                data = getattr(obj, field)
                if data:
                    return QVariant(data)
        return QVariant()

    def _dataIcon(self, obj, qidx):
        if qidx.column() != 0:
            return QVariant()

        try:
            thumbnail = obj.thumbnail
        except AttributeError:
            return QVariant()
        if thumbnail is NotLoaded:
            self.fillObj(obj, ['thumbnail'], qidx)
        elif thumbnail is NotAvailable:
            return QVariant()
        elif thumbnail.data is NotLoaded:
            self.fillObj(thumbnail, ['data'], qidx)
        elif thumbnail.data is NotAvailable:
            return QVariant()
        else:
            return QVariant(QIcon(QPixmap(QImage.fromData(thumbnail.data))))
        return QVariant()


class FilterTypeModel(QSortFilterProxyModel):
    def __init__(self, *args):
        super(FilterTypeModel, self).__init__(*args)
        self.types = []

    def setAcceptedTypes(self, types):
        self.types = types

    def filterAcceptsRow(self, row, parent_qidx):
        default = super(FilterTypeModel, self).filterAcceptsRow(row, parent_qidx)
        if not default:
            return False

        mdl = self.sourceModel()
        qidx = mdl.index(row, 0, parent_qidx)
        obj = mdl.data(qidx, ResultModel.RoleObject).value()
        actual = type(obj)
        for accepted in self.types:
            if issubclass(actual, accepted):
                return True
        return False



# -*- coding: utf-8 -*-

# Copyright(C) 2016 Julien Veyssier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QFrame

from weboob.applications.qbooblyrics.ui.songlyrics_ui import Ui_Songlyrics
from weboob.capabilities.base import empty


class Songlyrics(QFrame):
    def __init__(self, songlyrics, backend, parent=None):
        super(Songlyrics, self).__init__(parent)
        self.parent = parent
        self.ui = Ui_Songlyrics()
        self.ui.setupUi(self)

        self.songlyrics = songlyrics
        self.backend = backend

        self.ui.idEdit.setText(u'%s@%s' % (songlyrics.id, backend.name))
        if not empty(songlyrics.title):
            self.ui.titleLabel.setText(songlyrics.title)
        if not empty(songlyrics.artist):
            self.ui.artistLabel.setText(songlyrics.artist)
        else:
            self.ui.artistLabel.parent().hide()
        if not empty(songlyrics.content):
            self.ui.lyricsPlain.setPlainText(songlyrics.content)
        else:
            self.ui.lyricsPlain.parent().hide()

        self.ui.verticalLayout.setAlignment(Qt.AlignTop)
        self.ui.verticalLayout_2.setAlignment(Qt.AlignTop)




# -*- coding: utf-8 -*-

# Copyright(C) 2009-2014  Florent Fourcot
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


import hashlib
import time

from requests.exceptions import SSLError

from weboob.browser import LoginBrowser, URL, need_login
from weboob.exceptions import BrowserIncorrectPassword
from weboob.browser.exceptions import ServerError
from weboob.capabilities.bank import Account, AccountNotFound
from weboob.capabilities.base import find_object

from .pages import AccountsList, LoginPage, NetissimaPage, TitrePage, TitreHistory,\
    TransferPage, BillsPage, StopPage, TitreDetails, TitreValuePage, ASVHistory,\
    ASVInvest, DetailFondsPage, IbanPage, ActionNeededPage, ReturnPage, ProfilePage


__all__ = ['IngBrowser']


def start_with_main_site(f):
    def wrapper(*args, **kwargs):
        browser = args[0]

        if browser.url and browser.url.startswith('https://bourse.ingdirect.fr/'):
            for i in range(3):
                try:
                    browser.location('https://bourse.ingdirect.fr/priv/redirectIng.php?pageIng=COMPTE')
                except ServerError:
                    pass
                else:
                    break
            browser.where = 'start'
        elif browser.url and browser.url.startswith('https://ingdirectvie.ingdirect.fr/'):
            browser.lifeback.go()
            browser.where = 'start'

        return f(*args, **kwargs)
    return wrapper


class IngBrowser(LoginBrowser):
    BASEURL = 'https://secure.ingdirect.fr'
    TIMEOUT = 60.0

    # avoid relogin every time
    lifeback = URL(r'https://ingdirectvie.ingdirect.fr/b2b2c/entreesite/EntAccExit', ReturnPage)

    # Login and error
    loginpage = URL('/public/displayLogin.jsf.*', LoginPage)
    errorpage = URL('.*displayCoordonneesCommand.*', StopPage)
    actioneeded = URL('/general\?command=displayTRAlertMessage',
                      '/protected/pages/common/eco1/moveMoneyForbidden.jsf', ActionNeededPage)

    # CapBank
    accountspage = URL('/protected/pages/index.jsf',
                       '/protected/pages/asv/contract/(?P<asvpage>.*).jsf', AccountsList)
    titredetails = URL('/general\?command=display.*', TitreDetails)
    ibanpage = URL('/protected/pages/common/rib/initialRib.jsf', IbanPage)
    # CapBank-Market
    netissima = URL('/data/asv/fiches-fonds/fonds-netissima.html', NetissimaPage)
    starttitre = URL('/general\?command=goToAccount&zone=COMPTE', TitrePage)
    titrepage = URL('https://bourse.ingdirect.fr/priv/portefeuille-TR.php', TitrePage)
    titrehistory = URL('https://bourse.ingdirect.fr/priv/compte.php\?ong=3', TitreHistory)
    titrerealtime = URL('https://bourse.ingdirect.fr/streaming/compteTempsReelCK.php', TitrePage)
    titrevalue = URL('https://bourse.ingdirect.fr/priv/fiche-valeur.php\?val=(?P<val>.*)&pl=(?P<pl>.*)&popup=1', TitreValuePage)
    asv_history = URL('https://ingdirectvie.ingdirect.fr/b2b2c/epargne/CoeLisMvt',
                      'https://ingdirectvie.ingdirect.fr/b2b2c/epargne/CoeDetMvt', ASVHistory)
    asv_invest = URL('https://ingdirectvie.ingdirect.fr/b2b2c/epargne/CoeDetCon', ASVInvest)
    detailfonds = URL('https://ingdirectvie.ingdirect.fr/b2b2c/fonds/PerDesFac\?codeFonds=(.*)', DetailFondsPage)
    # CapDocument
    billpage = URL('/protected/pages/common/estatement/eStatement.jsf', BillsPage)
    # CapProfile
    profile = URL('/protected/pages/common/profil/(?P<page>\w+).jsf', ProfilePage)

    transfer = URL('/protected/pages/common/virement/index.jsf', TransferPage)

    __states__ = ['where']

    def __init__(self, *args, **kwargs):
        self.birthday = kwargs.pop('birthday')
        self.where = None
        LoginBrowser.__init__(self, *args, **kwargs)
        self.cache = {}
        self.cache["investments_data"] = {}

    def do_login(self):
        assert self.password.isdigit()
        assert self.birthday.isdigit()

        self.do_logout()
        self.loginpage.go()

        self.page.prelogin(self.username, self.birthday)
        self.page.login(self.password)
        if self.page.error():
            raise BrowserIncorrectPassword()
        if self.errorpage.is_here():
            raise BrowserIncorrectPassword('Please login on website to fill the form and retry')
        self.page.check_for_action_needed()

    @start_with_main_site
    def get_market_balance(self, account):
        if self.where != "start":
            self.accountspage.go()
            self.where = "start"

        data = self.get_investments_data(account)
        for i in range(5):
            if i > 0:
                self.logger.debug('Can\'t get market balance, retrying in %s seconds...', (2**i))
                time.sleep(2**i)
            if self.accountspage.go(data=data).has_link():
                break

        self.starttitre.go()
        self.where = u"titre"
        self.titrepage.go()
        self.titrerealtime.go()
        account.balance = self.page.get_balance() or account.balance
        self.cache["investments_data"][account.id] = self.page.doc or None

    @need_login
    @start_with_main_site
    def get_accounts_list(self, get_iban=True):
        self.accountspage.go()
        self.where = "start"

        for acc in self.page.get_list():
            if get_iban and acc.type in [Account.TYPE_CHECKING, Account.TYPE_SAVINGS]:
                self.go_account_page(acc)
                acc.iban = self.ibanpage.go().get_iban()

            if get_iban and acc.type in (Account.TYPE_MARKET, Account.TYPE_PEA):
                self.get_market_balance(acc)

            yield acc

    def get_account(self, _id):
        return find_object(self.get_accounts_list(get_iban=False), id=_id, error=AccountNotFound)

    def go_account_page(self, account):
        data = {"AJAX:EVENTS_COUNT": 1,
                "AJAXREQUEST": "_viewRoot",
                "ajaxSingle": "index:setAccount",
                "autoScroll": "",
                "index": "index",
                "index:setAccount": "index:setAccount",
                "javax.faces.ViewState": account._jid,
                "cptnbr": account._id
                }
        self.accountspage.go(data=data)
        self.where = "history"

    @need_login
    @start_with_main_site
    def get_coming(self, account):
        if account.type != Account.TYPE_CHECKING and\
                account.type != Account.TYPE_SAVINGS:
            raise NotImplementedError()
        account = self.get_account(account.id)
        self.go_account_page(account)
        jid = self.page.get_history_jid()
        if jid is None:
            self.logger.info('There is no history for this account')
            return

        return self.page.get_coming()

    @need_login
    @start_with_main_site
    def get_history(self, account):
        if account.type in (Account.TYPE_MARKET, Account.TYPE_PEA, Account.TYPE_LIFE_INSURANCE):
            for result in self.get_history_titre(account):
                yield result
            return

        elif account.type != Account.TYPE_CHECKING and\
                account.type != Account.TYPE_SAVINGS:
            raise NotImplementedError()

        account = self.get_account(account.id)
        self.go_account_page(account)
        jid = self.page.get_history_jid()
        if jid is None:
            self.logger.info('There is no history for this account')
            return

        if account.type == Account.TYPE_CHECKING:
            history_function = AccountsList.get_transactions_cc
            index = -1  # disable the index. It works without it on CC
        else:
            history_function = AccountsList.get_transactions_others
            index = 0
        hashlist = set()
        while True:
            i = index
            for transaction in history_function(self.page, index=index):
                transaction.id = hashlib.md5(transaction._hash).hexdigest()
                while transaction.id in hashlist:
                    transaction.id = hashlib.md5((transaction.id + "1").encode('ascii')).hexdigest()
                hashlist.add(transaction.id)
                i += 1
                yield transaction
            # if there is no more transactions, it is useless to continue
            if self.page.islast() or i == index:
                return
            if index >= 0:
                index = i
            data = {"AJAX:EVENTS_COUNT": 1,
                    "AJAXREQUEST": "_viewRoot",
                    "autoScroll": "",
                    "index": "index",
                    "index:%s:moreTransactions" % jid: "index:%s:moreTransactions" % jid,
                    "javax.faces.ViewState": account._jid
                    }
            self.accountspage.go(data=data)

    @need_login
    @start_with_main_site
    def iter_recipients(self, account):
        self.transfer.go()
        if not self.page.able_to_transfer(account):
            return iter([])
        self.page.go_to_recipient_selection(account)
        return self.page.get_recipients(origin=account)

    @need_login
    @start_with_main_site
    def init_transfer(self, account, recipient, transfer):
        self.transfer.go()
        self.page.do_transfer(account, recipient, transfer)
        return self.page.recap(account, recipient, transfer)

    @need_login
    @start_with_main_site
    def execute_transfer(self, transfer):
        self.page.confirm(self.password)
        return transfer

    def go_on_asv_detail(self, account, link):
        try:
            if self.page.asv_is_other:
                jid = self.page.get_asv_jid()
                data = {'index': "index", 'javax.faces.ViewState': jid, 'index:j_idcl': "index:asvInclude:goToAsvPartner"}
                self.accountspage.go(data=data)
            else:
                self.accountspage.go(asvpage="manageASVContract")
                self.page.submit()
            self.page.submit()
            self.location(link)

            return True
        except SSLError:
            return False

    def get_investments_data(self, account):
        return {"AJAX:EVENTS_COUNT": 1,
                "AJAXREQUEST": "_viewRoot",
                "ajaxSingle": "index:setAccount",
                "autoScroll": "",
                "index": "index",
                "index:setAccount": "index:setAccount",
                "javax.faces.ViewState": account._jid,
                "cptnbr": account._id
                }

    def go_investments(self, account):
        account = self.get_account(account.id)
        data = self.get_investments_data(account)

        # On ASV pages, data maybe not available.
        for i in range(5):
            if i > 0:
                self.logger.debug('Investments list empty, retrying in %s seconds...', (2**i))
                time.sleep(2**i)

                if i > 1:
                    self.do_logout()
                    self.do_login()
                    account = self.get_account(account.id)
                    data['cptnbr'] = account._id
                    data['javax.faces.ViewState'] = account._jid

            self.accountspage.go(data=data)

            if not self.page.has_error():
                break

        else:
            self.logger.warning("Unable to get investments list...")

        if self.page.is_asv:
            return

        self.starttitre.go()
        self.where = u"titre"
        self.titrepage.go()

    @need_login
    @start_with_main_site
    def get_investments(self, account):
        if account.type not in (Account.TYPE_MARKET, Account.TYPE_PEA, Account.TYPE_LIFE_INSURANCE):
            raise NotImplementedError()

        self.go_investments(account)

        if self.where == u'titre':
            if self.cache["investments_data"].get(account.id) == None:
                self.titrerealtime.go()
            for inv in self.page.iter_investments(account):
                yield inv
        elif self.page.asv_has_detail or account._jid:
            if self.go_on_asv_detail(account, '/b2b2c/epargne/CoeDetCon') is not False:
                self.where = u"asv"
                for inv in self.page.iter_investments():
                    yield inv

    def get_history_titre(self, account):
        self.go_investments(account)

        if self.where == u'titre':
            self.titrehistory.go()
        elif self.page.asv_has_detail or account._jid:
            if self.go_on_asv_detail(account, '/b2b2c/epargne/CoeLisMvt') is False:
                return iter([])
        else:
            return iter([])

        transactions = list()
        for tr in self.page.iter_history():
            transactions.append(tr)
        if self.asv_history.is_here():
            for tr in transactions:
                page = tr._detail.result().page if tr._detail else None
                tr.investments = list(page.get_investments()) if page and 'numMvt' in page.url else []
            self.lifeback.go()
        return iter(transactions)

    ############# CapDocument #############
    @start_with_main_site
    @need_login
    def get_subscriptions(self):
        self.billpage.go()
        if self.loginpage.is_here():
            self.do_login()
            return self.billpage.go().iter_account()
        else:
            return self.page.iter_account()

    @need_login
    def get_documents(self, subscription):
        self.billpage.go()
        data = {"AJAXREQUEST": "_viewRoot",
                "accountsel_form": "accountsel_form",
                subscription._formid: subscription._formid,
                "autoScroll": "",
                "javax.faces.ViewState": subscription._javax,
                "transfer_issuer_radio": subscription.id
                }
        self.billpage.go(data=data)
        return self.page.iter_documents(subid=subscription.id)

    def predownload(self, bill):
        self.page.postpredown(bill._localid)

    ############# CapProfile #############
    @start_with_main_site
    @need_login
    def get_profile(self):
        profile = self.profile.go(page='coordonnees').get_profile()
        self.profile.go(page='infosperso').update_profile(profile)
        return profile



# -*- coding: utf-8 -*-

# Copyright(C) 2011-2014 Laurent Bachelier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import re

from weboob.capabilities.paste import BasePaste, PasteNotFound
from weboob.browser.filters.standard import BrowserURL, CleanText, DateTime, Env, Field, RawText, Regexp
from weboob.browser.pages import HTMLPage
from weboob.browser.browsers import PagesBrowser
from weboob.browser.url import URL
from weboob.browser.elements import ItemElement, method
from weboob.exceptions import BrowserHTTPNotFound


class Spam(Exception):
    def __init__(self):
        super(Spam, self).__init__("Detected as spam and unable to handle the captcha")


class PastealaconPaste(BasePaste):
    # all pastes are public
    public = True

    # TODO perhaps move this logic elsewhere, remove this and id2url from capability
    # (page_url is required by pastoob)
    @property
    def page_url(self):
        return self.url


class PastePage(HTMLPage):
    @method
    class fill_paste(ItemElement):
        klass = PastealaconPaste

        obj_id = Env('id')
        obj_title = Regexp(CleanText('id("content")/h3'), r'Posted by (.+) on .+ \(')
        obj__date = DateTime(Regexp(CleanText('id("content")/h3'), r'Posted by .+ on (.+) \('))
        obj_contents = RawText('//textarea[@id="code"]')
        obj_url = BrowserURL('paste', id=Field('id'))

        def parse(self, el):
            # there is no 404, try to detect if there really is a content
            if len(el.xpath('id("content")/div[@class="syntax"]//ol')) != 1:
                raise PasteNotFound()


class CaptchaPage(HTMLPage):
    pass


class PostPage(HTMLPage):
    def post(self, paste, expiration=None):
        form = self.get_form(name='editor')
        form['code2'] = paste.contents
        form['poster'] = paste.title
        if expiration:
            form['expiry'] = expiration
        form.submit()


class PastealaconBrowser(PagesBrowser):
    BASEURL = 'http://paste.alacon.org/'

    paste = URL(r'(?P<id>\d+)', PastePage)
    captcha = URL(r'%s' % re.escape('pastebin.php?captcha=1'), CaptchaPage)
    raw = URL(r'%s(?P<id>\d+)' % re.escape('pastebin.php?dl='))
    post = URL(r'$', PostPage)

    @paste.id2url
    def get_paste(self, url):
        url = self.absurl(url, base=True)
        m = self.paste.match(url)
        if m:
            return PastealaconPaste(m.groupdict()['id'])

    def fill_paste(self, paste):
        """
        Get as much as information possible from the paste page
        """
        self.paste.stay_or_go(id=paste.id)
        return self.page.fill_paste(paste)

    def get_contents(self, _id):
        """
        Get the contents from the raw URL
        This is the fastest and safest method if you only want the content.
        Returns unicode.
        """
        try:
            # despite what the HTTP header says, it is iso8859
            return self.raw.open(id=_id).content.decode('iso8859-15')
        except BrowserHTTPNotFound:
            raise PasteNotFound()

    def post_paste(self, paste, expiration=None):
        self.post.stay_or_go().post(paste, expiration=expiration)
        if self.captcha.is_here():
            raise Spam()
        self.page.fill_paste(paste)



<%inherit file="layout.py"/>
from weboob.browser.pages import HTMLPage


class Page1(HTMLPage):
    def do_stuff(self, _id):
        raise NotImplementedError()


class Page2(HTMLPage):
    def do_more_stuff(self):
        raise NotImplementedError()



# -*- coding: utf-8 -*-

# Copyright(C) 2016      François Revol
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.capabilities.job import BaseJobAdvert
from weboob.browser.pages import HTMLPage
from weboob.browser.elements import ItemElement, ListElement, method
from weboob.browser.filters.standard import Regexp, CleanText, Date, Env, BrowserURL
from weboob.browser.filters.html import Link, CleanHTML
from weboob.tools.date import parse_french_date

class AdvertPage(HTMLPage):
    @method
    class get_job_advert(ItemElement):
        klass = BaseJobAdvert

        obj_id = Env('id')
        obj_url = BrowserURL('advert_page', id=Env('id'))
        obj_title = CleanText('//title')
        obj_job_name = CleanText('//title')
        obj_society_name = CleanText('//div[2]/div[@class="col-md-9"]/h4[1]')
        obj_publication_date = Date(CleanText('//div[2]/div[@class="col-md-9"]/small', replace=[(u'Ajoutée le', '')]), parse_func=parse_french_date)
        obj_place = Regexp(CleanText('//div[2]/div[@class="col-md-9"]/h4[2]'), '(.*) \(.*\)')
        obj_description = CleanHTML('//div[4]/div[@class="col-md-9"]')


class SearchPage(HTMLPage):
    @method
    class iter_job_adverts(ListElement):
        item_xpath = '//a[@class="list-group-item "]'

        class item(ItemElement):
            klass = BaseJobAdvert

            obj_id = Regexp(Link('.'), '.*fr/jobs/(\d+)/.*')
            obj_title = CleanText('h4/span[@class="job-title"]')
            obj_society_name = CleanText('h4/span[@class="job-company"]')
            obj_publication_date = Date(CleanText('h4/span[@class="badge pull-right"]'), parse_func=parse_french_date)



# -*- coding: utf-8 -*-

# Copyright(C) 2016      Edouard Lambert
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


import re, requests

from weboob.browser.pages import HTMLPage, LoggedPage, pagination
from weboob.browser.elements import ListElement, ItemElement, TableElement, method
from weboob.browser.filters.standard import CleanText, Date, CleanDecimal, Env, TableCell
from weboob.browser.filters.html import Link
from weboob.capabilities.bank import Account
from weboob.capabilities.profile import Profile
from weboob.capabilities.base import NotAvailable
from weboob.exceptions import BrowserPasswordExpired

from ..pages import Transaction


def MyDecimal(*args, **kwargs):
    kwargs.update(replace_dots=True, default=NotAvailable)
    return CleanDecimal(*args, **kwargs)


class LoginPage(HTMLPage):
    def get_error(self):
        return CleanText(default=False).filter(self.doc.xpath('//li[contains(@class, "erreur")]'))

    def login(self, login, password):
        form = self.get_form(id="myForm")
        form['login'] = login
        form['pwd'] = password
        form['mode'] = '1'
        form.submit()


class MovementsPage(LoggedPage, HTMLPage):
    def get_changecompte(self, link):
        form = self.get_form('//form[contains(@action, "changeCompte")]')
        m = re.search('\'(\d+).*\'(\d+)', link)
        form['perimetreMandatParentData'] = m.group(1)
        form['perimetreMandatEnfantData'] = m.group(2)
        # Can't do multi with async because of inconsistency...
        return self.browser.open(form.url, data=dict(form)).page, form.url, dict(form)

    @method
    class iter_accounts(ListElement):
        def parse(self, el):
            # multi accounts separated in multiple companies
            self.item_xpath = '//form[contains(@action, "changeCompte")]//ul[@class="listeEnfants"]/li/a'
            self.env['multi'] = bool(self.page.doc.xpath(self.item_xpath))
            if self.env['multi']:
                return

            # simple multi accounts
            self.item_xpath = '//form[contains(@action, "changeCompte")]//ul[@class="layerEnfants"]/li/a'
            self.env['multi'] = bool(self.page.doc.xpath(self.item_xpath))
            if self.env['multi']:
                return

            # single account
            self.item_xpath = '//*[@id="perimetreMandatEnfantLib"]'

        class item(ItemElement):
            klass = Account

            def obj_id(self):
                return CleanText('.')(self).strip().replace(' ', '').split('-')[0]

            def obj_label(self):
                return CleanText('.')(self).split("-")[-1].strip()

            obj_type = Account.TYPE_CHECKING

            obj_balance = Env('balance')
            obj_currency = Env('currency')
            obj__url = Env('url')
            obj__data = Env('data')

            def parse(self, el):
                page, url, data = self.page.get_changecompte(Link('.')(self)) if self.env['multi'] else (self.page, None, None)
                balance_xpath = '//div[contains(text(),"Solde")]/strong'
                self.env['balance'] = MyDecimal().filter(page.doc.xpath(balance_xpath))
                self.env['currency'] = Account.get_currency(CleanText().filter(page.doc.xpath(balance_xpath)))
                self.env['url'] = url
                self.env['data'] = data


    @pagination
    @method
    class iter_history(TableElement):
        item_xpath = '//table[@id="listeOperations" or @id="listeEffets"]/tbody/tr[td[5]]'
        head_xpath = '//table[@id="listeOperations"]/thead/tr/th'

        col_raw = re.compile(u'Libellé')
        col_debit = re.compile(u'Débit')
        col_credit = re.compile(u'Crédit')

        def next_page(self):
            url = Link('//a[contains(text(), "Page suivante")]', default=None)(self)
            if url:
                m = re.search('\s+\'([^\']+).*\'(\d+)', url)
                return requests.Request("POST", m.group(1), data={'numPage': m.group(2)})

        class item(ItemElement):
            klass = Transaction

            obj_raw = Transaction.Raw(TableCell('raw'))
            obj_date = Date(CleanText('./td[1]'), dayfirst=True)
            obj_vdate = Date(CleanText('./td[2]'), dayfirst=True)

            def obj_amount(self):
                credit = MyDecimal(TableCell('credit'))(self)
                debit = MyDecimal(TableCell('debit'))(self)
                return credit if credit else -debit


class ProfilePage(LoggedPage, HTMLPage):
    @method
    class get_profile(ItemElement):
        klass = Profile

        obj_name = CleanText('//div[@class="headerInfos"][not(@id)]/span')
        obj_phone = CleanText('//div[label[@id="tel_fix_label"]]/following-sibling::div[1]')
        obj_email = CleanText('//div[label[@id="email_label"]]/following-sibling::div[1]')


class PassExpiredPage(HTMLPage):
    def on_load(self):
        raise BrowserPasswordExpired("Renouvellement de mot de passe requis.")



# -*- coding: utf-8 -*-

# Copyright(C) 2011-2014 Laurent Bachelier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.capabilities.base import NotLoaded
from weboob.tools.backend import BackendConfig, Module
from weboob.tools.capabilities.paste import BasePasteModule
from weboob.tools.value import Value, ValueBackendPassword

from .browser import PastebinBrowser, PastebinPaste


class PastebinModule(Module, BasePasteModule):
    NAME = 'pastebin'
    MAINTAINER = u'Laurent Bachelier'
    EMAIL = 'laurent@bachelier.name'
    VERSION = '1.3'
    DESCRIPTION = 'Pastebin text sharing service'
    LICENSE = 'AGPLv3+'
    BROWSER = PastebinBrowser
    CONFIG = BackendConfig(
        Value('username', label='Optional username', default=''),
        ValueBackendPassword('password', label='Optional password', default=''),
        ValueBackendPassword('api_key',  label='Optional API key',  default='', noprompt=True),
    )

    EXPIRATIONS = {
        600: '10M',
        3600: '1H',
        3600 * 24: '1D',
        3600 * 24 * 30: '1M',
        False: 'N',
    }

    def create_default_browser(self):
        username = self.config['username'].get()
        if username:
            password = self.config['password'].get()
        else:
            password = None
        return self.create_browser(self.config['api_key'].get() or None,
                                   username, password)

    def new_paste(self, *args, **kwargs):
        return PastebinPaste(*args, **kwargs)

    def can_post(self, contents, title=None, public=None, max_age=None):
        if max_age is not None:
            if self.get_closest_expiration(max_age) is None:
                return 0
        if not title or len(title) <= 60:
            return 2
        return 1

    def get_paste(self, _id):
        return self.browser.get_paste(_id)

    def fill_paste(self, paste, fields):
        # if we only want the contents
        if fields == ['contents']:
            if paste.contents is NotLoaded:
                paste.contents = self.browser.get_contents(paste.id)
        # get all fields
        elif fields is None or len(fields):
            self.browser.fill_paste(paste)
        return paste

    def post_paste(self, paste, max_age=None, use_api=True):
        if max_age is not None:
            expiration = self.get_closest_expiration(max_age)
        else:
            expiration = None
        if use_api and self.config.get('api_key').get():
            self.browser.api_post_paste(paste, expiration=self.EXPIRATIONS.get(expiration))
        else:
            self.browser.post_paste(paste, expiration=self.EXPIRATIONS.get(expiration))

    OBJECTS = {PastebinPaste: fill_paste}



# -*- coding: utf-8 -*-

# Copyright(C) 2012-2017 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from collections import OrderedDict
from functools import wraps
import re

from weboob.capabilities.bank import CapBankTransferAddRecipient, AccountNotFound, Account, RecipientNotFound
from weboob.capabilities.contact import CapContact
from weboob.capabilities.profile import CapProfile
from weboob.capabilities.base import find_object
from weboob.tools.backend import Module, BackendConfig
from weboob.tools.value import Value, ValueBackendPassword

from .browser import CaisseEpargne, ChangeBrowser
from .cenet.browser import CenetBrowser

__all__ = ['CaisseEpargneModule']


def check_browser_type_iter(func):
    @wraps(func)
    def wrapper(self, *args, **kwargs):
        returned = False
        try:
            for obj in func(self, *args, **kwargs):
                yield obj
                returned = True
        except ChangeBrowser:
            assert not returned, 'cannot change browser type in the middle of the iteration'

            self.BROWSER = CenetBrowser
            self._browser = self.create_default_browser()
            for obj in func(self, *args, **kwargs):
                yield obj

    return wrapper


def check_browser_type(func):
    @wraps(func)
    def wrapper(self, *args, **kwargs):
        try:
            return func(self, *args, **kwargs)
        except ChangeBrowser:
            self.BROWSER = CenetBrowser
            self._browser = self.create_default_browser()
            return func(self, *args, **kwargs)

    return wrapper


class CaisseEpargneModule(Module, CapBankTransferAddRecipient, CapContact, CapProfile):
    NAME = 'caissedepargne'
    MAINTAINER = u'Romain Bignon'
    EMAIL = 'romain@weboob.org'
    VERSION = '1.3'
    DESCRIPTION = u'Caisse d\'Épargne'
    LICENSE = 'AGPLv3+'
    BROWSER = CaisseEpargne
    website_choices = OrderedDict([(k, u'%s (%s)' % (v, k)) for k, v in sorted({
        'www.caisse-epargne.fr':     u'Caisse d\'Épargne',
        'www.banquebcp.fr':          u'Banque BCP',
        }.items(), key=lambda k_v: (k_v[1], k_v[0]))])
    CONFIG = BackendConfig(Value('website',  label='Banque', choices=website_choices, default='www.caisse-epargne.fr'),
                           ValueBackendPassword('login',    label='Identifiant client', masked=False),
                           ValueBackendPassword('password', label='Code personnel', regexp='\d+'),
                           Value('nuser',                   label='User ID (optional)', default=''))

    def create_default_browser(self):
        return self.create_browser(nuser=self.config['nuser'].get(),
                                   username=self.config['login'].get(),
                                   password=self.config['password'].get(),
                                   domain=self.config['website'].get())

    @check_browser_type_iter
    def iter_accounts(self):
        for account in self.browser.get_accounts_list():
            yield account
        for account in self.browser.get_loans_list():
            yield account

    def get_account(self, _id):
        return find_object(self.iter_accounts(), id=_id, error=AccountNotFound)

    @check_browser_type_iter
    def iter_history(self, account):
        return self.browser.get_history(account)

    @check_browser_type_iter
    def iter_coming(self, account):
        return self.browser.get_coming(account)

    @check_browser_type_iter
    def iter_investment(self, account):
        return self.browser.get_investment(account)

    @check_browser_type_iter
    def iter_contacts(self):
        return self.browser.get_advisor()

    @check_browser_type
    def get_profile(self):
        return self.browser.get_profile()

    @check_browser_type_iter
    def iter_transfer_recipients(self, origin_account):
        if not isinstance(origin_account, Account):
            origin_account = find_object(self.iter_accounts(), id=origin_account, error=AccountNotFound)
        return self.browser.iter_recipients(origin_account)

    @check_browser_type
    def init_transfer(self, transfer, **params):
        self.logger.info('Going to do a new transfer')
        transfer.label = ' '.join(w for w in re.sub('[^0-9a-zA-Z/\-\?:\(\)\.,\'\+ ]+', '', transfer.label).split()).upper()
        if transfer.account_iban:
            account = find_object(self.iter_accounts(), iban=transfer.account_iban, error=AccountNotFound)
        else:
            account = find_object(self.iter_accounts(), id=transfer.account_id, error=AccountNotFound)

        if transfer.recipient_iban:
            recipient = find_object(self.iter_transfer_recipients(account.id), iban=transfer.recipient_iban, error=RecipientNotFound)
        else:
            recipient = find_object(self.iter_transfer_recipients(account.id), id=transfer.recipient_id, error=RecipientNotFound)

        return self.browser.init_transfer(account, recipient, transfer)

    @check_browser_type
    def execute_transfer(self, transfer, **params):
        return self.browser.execute_transfer(transfer)

    @check_browser_type
    def new_recipient(self, recipient, **params):
        #recipient.label = ' '.join(w for w in re.sub('[^0-9a-zA-Z:\/\-\?\(\)\.,\'\+ ]+', '', recipient.label).split())
        return self.browser.new_recipient(recipient, **params)



#!/usr/bin/env python3
import os
import re
import subprocess
import sys

import configparser

since = sys.argv[1]


def changed_modules(changes, changetype):
    for change in changes:
        change = change.decode('utf-8').split()
        if change[0] == changetype:
            m = re.match('modules/([^/]+)/__init__\.py', change[1])
            if m:
                yield m.group(1)


def get_caps(module, config):
    try:
        return sorted(c for c in config[module]['capabilities'].split() if c != 'CapCollection')
    except KeyError:
        return ['**** FILL ME **** (running weboob update could help)']

os.chdir(os.path.join(os.path.dirname(__file__), os.path.pardir))
modules_info = configparser.ConfigParser()
with open('modules/modules.list') as f:
    modules_info.read_file(f)
git_cmd = ['git', 'diff', '--no-renames', '--name-status', '%s..HEAD' % since, '--', 'modules/']

added_modules = sorted(changed_modules(subprocess.check_output(git_cmd).splitlines(), 'A'))
deleted_modules = sorted(changed_modules(subprocess.check_output(git_cmd).splitlines(), 'D'))

for added_module in added_modules:
    print('        * New %s module (%s)' % (added_module, ', '.join(get_caps(added_module, modules_info))))
for deleted_module in deleted_modules:
    print('        * Deleted %s module' % deleted_module)



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2014 Romain Bignon, Florent Fourcot
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import re
from datetime import timedelta

from weboob.capabilities.bank import CapBankTransfer, Account, AccountNotFound, RecipientNotFound
from weboob.capabilities.bill import CapDocument, Bill, Subscription,\
    SubscriptionNotFound, DocumentNotFound
from weboob.capabilities.profile import CapProfile
from weboob.capabilities.base import find_object, NotAvailable
from weboob.tools.backend import Module, BackendConfig
from weboob.tools.value import ValueBackendPassword, ValueDate
from weboob.browser.exceptions import ServerError

from .browser import IngBrowser

__all__ = ['INGModule']


class INGModule(Module, CapBankTransfer, CapDocument, CapProfile):
    NAME = 'ing'
    MAINTAINER = u'Florent Fourcot'
    EMAIL = 'weboob@flo.fourcot.fr'
    VERSION = '1.3'
    LICENSE = 'AGPLv3+'
    DESCRIPTION = 'ING Direct'
    CONFIG = BackendConfig(ValueBackendPassword('login',
                                                label=u'Numéro client',
                                                masked=False),
                           ValueBackendPassword('password',
                                                label='Code secret',
                                                regexp='^(\d{6})$'),
                           ValueDate('birthday',
                                     label='Date de naissance',
                                     formats=('%d%m%Y', '%d/%m/%Y', '%d-%m-%Y'))
                           )
    BROWSER = IngBrowser

    def create_default_browser(self):
        return self.create_browser(self.config['login'].get(),
                                   self.config['password'].get(),
                                   birthday=self.config['birthday'].get())

    def iter_resources(self, objs, split_path):
        if Account in objs:
            self._restrict_level(split_path)
            return self.iter_accounts()
        if Subscription in objs:
            self._restrict_level(split_path)
            return self.iter_subscription()

    def iter_accounts(self):
        return self.browser.get_accounts_list()

    def get_account(self, _id):
        return self.browser.get_account(_id)

    def iter_history(self, account):
        if not isinstance(account, Account):
            account = self.get_account(account)
        return self.browser.get_history(account)

    def iter_transfer_recipients(self, account):
        if not isinstance(account, Account):
            account = self.get_account(account)
        return self.browser.iter_recipients(account)

    def init_transfer(self, transfer, **params):
        self.logger.info('Going to do a new transfer')
        transfer.label = ' '.join(w for w in re.sub('[^0-9a-zA-Z/\-\?:\(\)\.,\'\+ ]+', '', transfer.label).split()).upper()
        if transfer.account_iban:
            account = find_object(self.iter_accounts(), iban=transfer.account_iban, error=AccountNotFound)
        else:
            account = find_object(self.iter_accounts(), id=transfer.account_id, error=AccountNotFound)

        if transfer.recipient_iban:
            recipient = find_object(self.iter_transfer_recipients(account.id), iban=transfer.recipient_iban, error=RecipientNotFound)
        else:
            recipient = find_object(self.iter_transfer_recipients(account.id), id=transfer.recipient_id, error=RecipientNotFound)

        return self.browser.init_transfer(account, recipient, transfer)

    def execute_transfer(self, transfer, **params):
        return self.browser.execute_transfer(transfer)

    def transfer_check_exec_date(self, old_exec_date, new_exec_date):
        return old_exec_date <= new_exec_date <= old_exec_date + timedelta(days=4)

    def iter_investment(self, account):
        if not isinstance(account, Account):
            account = self.get_account(account)
        return self.browser.get_investments(account)

    def iter_coming(self, account):
        if not isinstance(account, Account):
            account = self.get_account(account)
        return self.browser.get_coming(account)

    def iter_subscription(self):
        return self.browser.get_subscriptions()

    def get_subscription(self, _id):
        return find_object(self.browser.get_subscriptions(), id=_id, error=SubscriptionNotFound)

    def get_document(self, _id):
        subscription = self.get_subscription(_id.split('-')[0])
        return find_object(self.browser.get_documents(subscription), id=_id, error=DocumentNotFound)

    def iter_documents(self, subscription):
        if not isinstance(subscription, Subscription):
            subscription = self.get_subscription(subscription)
        return self.browser.get_documents(subscription)

    def download_document(self, bill):
        if not isinstance(bill, Bill):
            bill = self.get_document(bill)
        self.get_document(bill.id)
        try:
            self.browser.predownload(bill)
        except ServerError:
            return NotAvailable
        assert(self.browser.response.headers['content-type'] in ["application/pdf", "application/download"])
        return self.browser.response.content

    def get_profile(self):
        return self.browser.get_profile()



# -*- coding: utf-8 -*-

# Copyright(C) 2015      Vincent A
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.tools.backend import Module, BackendConfig
from weboob.tools.value import Value, ValueBackendPassword
from weboob.capabilities.collection import CapCollection, CollectionNotFound
from weboob.capabilities.video import CapVideo, BaseVideo

from .browser import FunmoocBrowser

__all__ = ['FunmoocModule']


class FunmoocModule(Module, CapVideo, CapCollection):
    NAME = 'funmooc'
    DESCRIPTION = u'France-Université-Numérique MOOC website'
    MAINTAINER = u'Vincent A'
    EMAIL = 'dev@indigo.re'
    LICENSE = 'AGPLv3+'
    VERSION = '1.3'

    CONFIG = BackendConfig(Value('email', label='Email'),
                           ValueBackendPassword('password', label='Password'),
                           Value('quality', label='Quality', default='HD',
                                 choices=['HD', 'SD', 'LD']))

    BROWSER = FunmoocBrowser

    def create_default_browser(self):
        quality = self.config['quality'].get().upper()
        return self.create_browser(self.config['email'].get(),
                                   self.config['password'].get(),
                                   quality=quality)

    def get_video(self, _id):
        return self.browser.get_video(_id)

    def iter_resources(self, objs, split_path):
        if len(split_path) == 0:
            return self.browser.iter_courses()
        elif len(split_path) == 1:
            return self.browser.iter_chapters(*split_path)
        elif len(split_path) == 2:
            return self.browser.iter_sections(*split_path)
        elif len(split_path) == 3:
            return self.browser.iter_videos(*split_path)

    def _matches(self, title, pattern):
        title = title.lower()
        words = pattern.lower().split()
        return all(word in title for word in words)

    def search_videos(self, pattern, sortby=0, nsfw=False):
        queue = [[]]
        while len(queue):
            path = queue.pop()
            for item in self.iter_resources(BaseVideo, path):
                if isinstance(item, BaseVideo):
                    if self._matches(item.title, pattern):
                        yield item
                else: # collection
                    newpath = item.split_path
                    if self._matches(item.title, pattern):
                        self.logger.debug('%s matches, returning content',
                                          item.title)
                        for item in self.iter_resources_flat(BaseVideo, newpath):
                            yield item
                        return
                    queue.append(newpath)

    def validate_collection(self, objs, collection):
        if not self.browser.check_collection(collection.split_path):
            raise CollectionNotFound()



# -*- coding: utf-8 -*-

# Copyright(C) 2015 Guilhem Bonnefille
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.browser import PagesBrowser, URL
from weboob.browser.exceptions import HTTPNotFound
from .pages import RmllCollectionPage, RmllVideoPage, RmllChannelsPage, RmllSearchPage, RmllLatestPage, RmllDurationPage

__all__ = ['RmllBrowser']


class RmllBrowser(PagesBrowser):
    BASEURL = 'https://rmll.ubicast.tv'

    index_page = URL(r'channels/content/(?P<id>.+)', RmllCollectionPage)
    latest_page = URL(r'api/v2/latest/', RmllLatestPage)
    video_page = URL(r'permalink/(?P<id>.+)/', RmllVideoPage)
    channels_page = URL(r'api/v2/channels/content/\?parent_oid=(?P<oid>.*)', RmllChannelsPage)
    search_page = URL(r'api/v2/search/\?search=(?P<pattern>.+)', RmllSearchPage)
    duration_page = URL(r'api/v2/medias/modes/\?oid=(?P<oid>.*)', RmllDurationPage)

    def __init__(self, *args, **kwargs):
        self.channels = None
        PagesBrowser.__init__(self, *args, **kwargs)

    @video_page.id2url
    def get_video(self, url, video=None):
        self.location(url)
        assert self.video_page.is_here()
        video = self.page.get_video(obj=video)
        video.duration = self.duration_page.go(oid=video.id).get_duration()
        return video

    def search_videos(self, pattern):
        url = self.search_page.build(pattern=pattern)
        self.location(url)
        return self.page.iter_resources()

    def get_latest_videos(self):
        url = self.latest_page.build()
        self.location(url)
        assert self.latest_page.is_here()
        return self.page.iter_resources()

    def get_channel_videos(self, split_path):
        oid = ''
        if len(split_path) > 0:
            oid = split_path[-1]
        try:
            url = self.channels_page.build(oid=oid)
            self.location(url)
            assert self.channels_page.is_here()
            for video in self.page.iter_resources(split_path):
                yield video

        except HTTPNotFound:
            pass
